/*
Copyright (c) 2007 Rodrigo Roman and Ramon Roman (rrc2soft)

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of "rrc2soft" nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// Includes
#include <PA9.h>       // Include for PA_Lib

#include "ct_const.h"
#include "ct_linkedlist.h"
#include "ct_utillib.h"

#include "gfx/all_gfx.h" // PAGfx Include

const char gfx_pinyin[20900][8] = {
{"yi1"},
{"ding1"},
{"----"},
{"qi1"},
{"----"},
{"----"},
{"----"},
{"wan4"},
{"zhang4"},
{"san1"},
{"shang4"},
{"xia4"},
{"ji1"},
{"bu4"},
{"yu3"},
{"----"},
{"gai4"},
{"chou3"},
{"----"},
{"zhuan1"},
{"qie3"},
{"pi1"},
{"shi4"},
{"----"},
{"qiu1"},
{"bing3"},
{"ye4"},
{"cong2"},
{"dong1"},
{"si1"},
{"cheng2"},
{"----"},
{"----"},
{"----"},
{"diu1"},
{"----"},
{"liang3"},
{"yan2"},
{"----"},
{"sang4"},
{"----"},
{"----"},
{"ge4"},
{"ya1"},
{"----"},
{"zhong1"},
{"----"},
{"----"},
{"feng1"},
{"----"},
{"chuan4"},
{"----"},
{"lin2"},
{"----"},
{"----"},
{"----"},
{"wan2"},
{"dan1"},
{"wei4"},
{"zhu3"},
{"----"},
{"li4"},
{"ju3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nai3"},
{"----"},
{"jiu3"},
{"----"},
{"----"},
{"ma5"},
{"yi4"},
{"----"},
{"zhi1"},
{"wu1"},
{"zha4"},
{"hu1"},
{"fa2"},
{"yue4"},
{"----"},
{"ping1"},
{"pang1"},
{"qiao2"},
{"----"},
{"guai1"},
{"----"},
{"cheng2"},
{"yi3"},
{"----"},
{"----"},
{"mie1"},
{"jiu3"},
{"qi3"},
{"ye3"},
{"xi2"},
{"xiang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu1"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mai3"},
{"luan4"},
{"----"},
{"ru3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"le5"},
{"----"},
{"yu3"},
{"zheng1"},
{"----"},
{"shi4"},
{"er4"},
{"chu4"},
{"yu2"},
{"kui1"},
{"----"},
{"yun2"},
{"hu4"},
{"qi2"},
{"wu3"},
{"jing3"},
{"----"},
{"----"},
{"gen4"},
{"----"},
{"ya4"},
{"xie1"},
{"----"},
{"----"},
{"----"},
{"qi4"},
{"----"},
{"wang2"},
{"kang4"},
{"----"},
{"jiao1"},
{"hai4"},
{"yi4"},
{"chan3"},
{"heng1"},
{"mu3"},
{"----"},
{"xiang3"},
{"jing1"},
{"ting2"},
{"liang4"},
{"----"},
{"----"},
{"----"},
{"qin1"},
{"bo2"},
{"----"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ren2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"shen2"},
{"ren2"},
{"le4"},
{"ding1"},
{"ze4"},
{"jin3"},
{"pu1"},
{"chou2"},
{"----"},
{"zhang3"},
{"jin1"},
{"jie4"},
{"----"},
{"reng2"},
{"cong2"},
{"----"},
{"----"},
{"lun2"},
{"----"},
{"cang1"},
{"zi3"},
{"shi4"},
{"ta1"},
{"zhang4"},
{"fu4"},
{"xian1"},
{"----"},
{"----"},
{"----"},
{"tong2"},
{"ren4"},
{"qian1"},
{"----"},
{"yi4"},
{"----"},
{"dai4"},
{"ling4"},
{"yi3"},
{"----"},
{"----"},
{"sa1"},
{"----"},
{"yi2"},
{"----"},
{"men5"},
{"----"},
{"----"},
{"----"},
{"yang3"},
{"----"},
{"zhong4"},
{"pi3"},
{"----"},
{"wu3"},
{"jian4"},
{"jia4"},
{"----"},
{"----"},
{"----"},
{"ren4"},
{"----"},
{"fen4"},
{"----"},
{"fang3"},
{"----"},
{"qi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kang4"},
{"yi1"},
{"----"},
{"----"},
{"wu3"},
{"ji4"},
{"fu2"},
{"fa2"},
{"xiu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhong4"},
{"you1"},
{"huo3"},
{"hui4"},
{"yu3"},
{"----"},
{"----"},
{"san3"},
{"wei3"},
{"chuan2"},
{"----"},
{"ya2"},
{"----"},
{"shang1"},
{"chang1"},
{"lun2"},
{"cang1"},
{"----"},
{"----"},
{"wei3"},
{"zhu4"},
{"----"},
{"----"},
{"----"},
{"bo2"},
{"gu1"},
{"----"},
{"----"},
{"----"},
{"ban4"},
{"----"},
{"ling2"},
{"----"},
{"shen1"},
{"----"},
{"si4"},
{"----"},
{"si4"},
{"qie2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian4"},
{"----"},
{"----"},
{"dan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wei4"},
{"di1"},
{"zhu4"},
{"zuo3"},
{"you4"},
{"----"},
{"ti3"},
{"----"},
{"he2"},
{"----"},
{"tuo2"},
{"she2"},
{"yu2"},
{"yi4"},
{"fo2"},
{"zuo4"},
{"----"},
{"ning4"},
{"tong2"},
{"ni3"},
{"----"},
{"----"},
{"yong4"},
{"wa3"},
{"qian1"},
{"----"},
{"ka3"},
{"----"},
{"pei4"},
{"----"},
{"----"},
{"lao3"},
{"----"},
{"----"},
{"yang2"},
{"bai3"},
{"----"},
{"----"},
{"jia1"},
{"er4"},
{"----"},
{"ji2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tiao1"},
{"jiao3"},
{"----"},
{"yi4"},
{"shi3"},
{"----"},
{"----"},
{"----"},
{"kan3"},
{"zhi2"},
{"----"},
{"----"},
{"----"},
{"chi3"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"shi4"},
{"----"},
{"zhu1"},
{"----"},
{"you4"},
{"----"},
{"----"},
{"mou2"},
{"----"},
{"----"},
{"tong2"},
{"----"},
{"----"},
{"----"},
{"gong4"},
{"----"},
{"yi1"},
{"----"},
{"----"},
{"xia2"},
{"----"},
{"----"},
{"lu:3"},
{"----"},
{"jiao3"},
{"zhen1"},
{"ce4"},
{"qiao2"},
{"kuai4"},
{"chai2"},
{"----"},
{"nong2"},
{"----"},
{"wu3"},
{"hou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bian4"},
{"----"},
{"----"},
{"----"},
{"cu4"},
{"e2"},
{"qiu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jun4"},
{"----"},
{"----"},
{"----"},
{"zu3"},
{"qiao4"},
{"li4"},
{"yong3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"su2"},
{"fu2"},
{"----"},
{"li3"},
{"----"},
{"ping1"},
{"bao3"},
{"yu2"},
{"si4"},
{"----"},
{"xin4"},
{"----"},
{"yu3"},
{"----"},
{"----"},
{"chou2"},
{"----"},
{"yan3"},
{"liang3"},
{"li4"},
{"----"},
{"----"},
{"jian3"},
{"xiu1"},
{"fu3"},
{"----"},
{"ju1"},
{"----"},
{"pai2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"feng4"},
{"----"},
{"an3"},
{"----"},
{"----"},
{"----"},
{"bi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"guan1"},
{"bei4"},
{"----"},
{"shu1"},
{"----"},
{"----"},
{"dao3"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"tang3"},
{"hou4"},
{"yi3"},
{"----"},
{"ti4"},
{"----"},
{"----"},
{"jie4"},
{"----"},
{"chang4"},
{"----"},
{"----"},
{"----"},
{"kong3"},
{"juan4"},
{"----"},
{"ju4"},
{"qian4"},
{"ni2"},
{"----"},
{"zhuo1"},
{"wo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhai4"},
{"----"},
{"zhi2"},
{"----"},
{"qing1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan3"},
{"----"},
{"----"},
{"----"},
{"jia3"},
{"jie2"},
{"----"},
{"----"},
{"----"},
{"ruo4"},
{"----"},
{"wei1"},
{"pian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zuo4"},
{"----"},
{"ting2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zong3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ou3"},
{"tou1"},
{"----"},
{"----"},
{"----"},
{"lou2"},
{"----"},
{"----"},
{"fen4"},
{"chang2"},
{"gui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu4"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dai3"},
{"----"},
{"tang3"},
{"----"},
{"bin1"},
{"chu3"},
{"nuo2"},
{"----"},
{"----"},
{"cui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chi4"},
{"sha3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"liao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiu4"},
{"seng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiao1"},
{"jian4"},
{"tong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"jiang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jing3"},
{"xuan1"},
{"----"},
{"----"},
{"----"},
{"dan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ru2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lei3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"er2"},
{"wu4"},
{"yun3"},
{"----"},
{"yuan2"},
{"xiong1"},
{"chong1"},
{"zhao4"},
{"----"},
{"xian1"},
{"guang1"},
{"----"},
{"ke4"},
{"----"},
{"mian3"},
{"----"},
{"----"},
{"----"},
{"dui4"},
{"----"},
{"----"},
{"tu4"},
{"si4"},
{"yan3"},
{"----"},
{"----"},
{"----"},
{"dang3"},
{"----"},
{"dou1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jing1"},
{"----"},
{"----"},
{"ru4"},
{"----"},
{"----"},
{"quan2"},
{"----"},
{"----"},
{"ba1"},
{"gong1"},
{"liu4"},
{"xi1"},
{"----"},
{"lan2"},
{"gong4"},
{"----"},
{"guan1"},
{"xing1"},
{"bing1"},
{"qi2"},
{"ju4"},
{"dian3"},
{"zi1"},
{"----"},
{"yang3"},
{"jian1"},
{"shou4"},
{"----"},
{"----"},
{"ji4"},
{"chan3"},
{"----"},
{"----"},
{"----"},
{"nei4"},
{"----"},
{"----"},
{"gang1"},
{"ran3"},
{"----"},
{"----"},
{"ce4"},
{"zai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mao4"},
{"----"},
{"----"},
{"mian3"},
{"----"},
{"rong3"},
{"----"},
{"xie3"},
{"----"},
{"jun1"},
{"nong2"},
{"----"},
{"----"},
{"----"},
{"guan1"},
{"----"},
{"zhong3"},
{"----"},
{"yuan1"},
{"ming2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dong1"},
{"----"},
{"----"},
{"feng2"},
{"bing1"},
{"hu4"},
{"chong1"},
{"jue2"},
{"----"},
{"kuang4"},
{"ye3"},
{"leng3"},
{"----"},
{"----"},
{"----"},
{"dong4"},
{"xian3"},
{"lie4"},
{"----"},
{"----"},
{"jing4"},
{"----"},
{"----"},
{"----"},
{"qi1"},
{"----"},
{"zhun3"},
{"----"},
{"----"},
{"liang2"},
{"----"},
{"diao1"},
{"ling2"},
{"----"},
{"----"},
{"jian3"},
{"----"},
{"cou4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lin3"},
{"----"},
{"ning2"},
{"----"},
{"----"},
{"ji1"},
{"fan2"},
{"----"},
{"----"},
{"feng4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"ping2"},
{"----"},
{"kai3"},
{"huang2"},
{"----"},
{"----"},
{"deng4"},
{"----"},
{"qu1"},
{"xiong1"},
{"----"},
{"tu1"},
{"ao1"},
{"chu1"},
{"ji1"},
{"----"},
{"han2"},
{"----"},
{"zao2"},
{"dao1"},
{"diao1"},
{"----"},
{"ren4"},
{"----"},
{"----"},
{"fen1"},
{"qie1"},
{"yi4"},
{"----"},
{"kan1"},
{"----"},
{"----"},
{"chu2"},
{"wen3"},
{"----"},
{"----"},
{"xing2"},
{"hua2"},
{"----"},
{"----"},
{"----"},
{"yue4"},
{"lie4"},
{"liu2"},
{"ze2"},
{"gang1"},
{"chuang4"},
{"----"},
{"chu1"},
{"----"},
{"----"},
{"shan1"},
{"----"},
{"----"},
{"----"},
{"pan4"},
{"----"},
{"----"},
{"----"},
{"bao4"},
{"li4"},
{"----"},
{"bie2"},
{"----"},
{"jing3"},
{"gua1"},
{"----"},
{"dao4"},
{"----"},
{"----"},
{"ku1"},
{"----"},
{"----"},
{"zhi4"},
{"shua1"},
{"quan4"},
{"sha1"},
{"ci4"},
{"ke4"},
{"----"},
{"gui4"},
{"----"},
{"gui4"},
{"kai3"},
{"duo4"},
{"ji4"},
{"ti4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xue1"},
{"----"},
{"la4"},
{"qian2"},
{"----"},
{"----"},
{"gua3"},
{"jian4"},
{"----"},
{"----"},
{"ti1"},
{"----"},
{"pou1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wan1"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"yan3"},
{"----"},
{"----"},
{"----"},
{"bo1"},
{"----"},
{"ju4"},
{"----"},
{"sheng4"},
{"jian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu4"},
{"----"},
{"----"},
{"ge1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"piao1"},
{"----"},
{"jiao3"},
{"----"},
{"qiao1"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"quan4"},
{"ban4"},
{"gong1"},
{"jia1"},
{"wu4"},
{"mai4"},
{"lie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dong4"},
{"zhu4"},
{"nu3"},
{"jie2"},
{"qu2"},
{"shao4"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"jin4"},
{"lao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"he2"},
{"shi4"},
{"----"},
{"----"},
{"----"},
{"bo2"},
{"----"},
{"----"},
{"----"},
{"yong3"},
{"----"},
{"mian3"},
{"----"},
{"xun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"le4"},
{"----"},
{"----"},
{"----"},
{"xu4"},
{"----"},
{"kan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qin2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shao2"},
{"----"},
{"----"},
{"----"},
{"gou1"},
{"wu4"},
{"yun2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bao1"},
{"cong1"},
{"----"},
{"xiong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pu2"},
{"----"},
{"pao2"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi3"},
{"hua4"},
{"bei3"},
{"----"},
{"chi2"},
{"fang1"},
{"----"},
{"----"},
{"za1"},
{"----"},
{"----"},
{"jiang4"},
{"kuang1"},
{"----"},
{"xia2"},
{"----"},
{"----"},
{"gui3"},
{"----"},
{"----"},
{"----"},
{"fei3"},
{"----"},
{"----"},
{"----"},
{"kui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi3"},
{"qu1"},
{"yi1"},
{"----"},
{"----"},
{"bian3"},
{"ni4"},
{"----"},
{"shi2"},
{"----"},
{"qian1"},
{"----"},
{"sa4"},
{"----"},
{"sheng1"},
{"wu3"},
{"hui4"},
{"ban4"},
{"----"},
{"----"},
{"----"},
{"hua2"},
{"xie2"},
{"----"},
{"bei1"},
{"zu2"},
{"zhuo2"},
{"----"},
{"dan1"},
{"mai4"},
{"nan2"},
{"----"},
{"----"},
{"bo2"},
{"----"},
{"bu3"},
{"----"},
{"bian4"},
{"----"},
{"zhan4"},
{"ka3"},
{"lu2"},
{"you3"},
{"lu3"},
{"----"},
{"gua4"},
{"wo4"},
{"----"},
{"----"},
{"----"},
{"wei4"},
{"----"},
{"----"},
{"zhi1"},
{"mao3"},
{"yin4"},
{"wei1"},
{"----"},
{"ji2"},
{"que4"},
{"luan3"},
{"----"},
{"juan4"},
{"xie4"},
{"----"},
{"jin3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qing1"},
{"----"},
{"----"},
{"chang3"},
{"----"},
{"e4"},
{"ting1"},
{"li4"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"ya1"},
{"yan4"},
{"she4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ce4"},
{"----"},
{"----"},
{"li2"},
{"----"},
{"hou4"},
{"----"},
{"----"},
{"cuo4"},
{"----"},
{"yuan2"},
{"----"},
{"----"},
{"xiang1"},
{"yan3"},
{"----"},
{"jue2"},
{"xia4"},
{"----"},
{"chu2"},
{"jiu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qu4"},
{"----"},
{"----"},
{"----"},
{"xian4"},
{"----"},
{"san1"},
{"can1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"you4"},
{"cha1"},
{"ji2"},
{"you3"},
{"shuang1"},
{"fan3"},
{"----"},
{"----"},
{"----"},
{"fa1"},
{"----"},
{"----"},
{"shu1"},
{"----"},
{"qu3"},
{"shou4"},
{"bian4"},
{"xu4"},
{"----"},
{"pan4"},
{"----"},
{"----"},
{"----"},
{"sou3"},
{"die2"},
{"----"},
{"----"},
{"kou3"},
{"gu3"},
{"ju4"},
{"ling4"},
{"----"},
{"dao1"},
{"kou4"},
{"zhi3"},
{"jiao4"},
{"zhao4"},
{"ba1"},
{"ding1"},
{"ke3"},
{"tai2"},
{"chi4"},
{"shi3"},
{"you4"},
{"----"},
{"po3"},
{"ye4"},
{"hao4"},
{"si1"},
{"tan4"},
{"----"},
{"le4"},
{"diao1"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"chi1"},
{"ge4"},
{"----"},
{"yao1"},
{"----"},
{"he2"},
{"ji2"},
{"diao4"},
{"----"},
{"tong2"},
{"ming2"},
{"hou4"},
{"li4"},
{"tu3"},
{"xiang4"},
{"zha4"},
{"xia4"},
{"----"},
{"lu:3"},
{"----"},
{"ma5"},
{"----"},
{"----"},
{"----"},
{"jun1"},
{"----"},
{"lin4"},
{"tun1"},
{"yin2"},
{"fei4"},
{"bi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fou3"},
{"ba5"},
{"dun1"},
{"fen1"},
{"----"},
{"han2"},
{"ting1"},
{"hang2"},
{"shun3"},
{"qi3"},
{"----"},
{"zi1"},
{"----"},
{"----"},
{"wu2"},
{"chao3"},
{"----"},
{"----"},
{"xi1"},
{"chui1"},
{"----"},
{"wen3"},
{"hou3"},
{"----"},
{"wu2"},
{"----"},
{"ya5"},
{"----"},
{"----"},
{"e4"},
{"----"},
{"----"},
{"dai1"},
{"----"},
{"cheng2"},
{"----"},
{"gao4"},
{"fu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ne5"},
{"----"},
{"fu3"},
{"yi4"},
{"----"},
{"ou3"},
{"li4"},
{"bai4"},
{"yuan2"},
{"----"},
{"----"},
{"qiang4"},
{"wu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ne5"},
{"----"},
{"ling2"},
{"----"},
{"you1"},
{"----"},
{"zhou1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gu1"},
{"----"},
{"wei4"},
{"----"},
{"he1"},
{"nao2"},
{"xia1"},
{"pei1"},
{"----"},
{"----"},
{"shen1"},
{"hu1"},
{"ming4"},
{"----"},
{"----"},
{"ju3"},
{"----"},
{"za1"},
{"----"},
{"duo1"},
{"----"},
{"pao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zha4"},
{"he2"},
{"----"},
{"jiu4"},
{"yong3"},
{"fu4"},
{"----"},
{"zhou4"},
{"----"},
{"----"},
{"gu1"},
{"ka1"},
{"----"},
{"----"},
{"long2"},
{"dong1"},
{"ning2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi2"},
{"lie3"},
{"zi1"},
{"mie1"},
{"mi1"},
{"zhi3"},
{"yao3"},
{"----"},
{"----"},
{"luo4"},
{"----"},
{"zan2"},
{"----"},
{"ke2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xian2"},
{"----"},
{"----"},
{"xiu1"},
{"----"},
{"yan1"},
{"----"},
{"yi1"},
{"ai1"},
{"pin3"},
{"shen3"},
{"----"},
{"hong1"},
{"----"},
{"duo1"},
{"wa1"},
{"ha1"},
{"zai1"},
{"----"},
{"----"},
{"----"},
{"xiang3"},
{"ai1"},
{"----"},
{"----"},
{"ya1"},
{"da1"},
{"xiao1"},
{"bi4"},
{"----"},
{"----"},
{"hua1"},
{"----"},
{"kuai4"},
{"----"},
{"----"},
{"ji4"},
{"nong2"},
{"mou1"},
{"yo5"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ge1"},
{"e2"},
{"----"},
{"shao4"},
{"li1"},
{"na3"},
{"----"},
{"----"},
{"ku1"},
{"xiao1"},
{"----"},
{"----"},
{"----"},
{"zhe2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bu3"},
{"----"},
{"heng1"},
{"geng3"},
{"----"},
{"ge3"},
{"----"},
{"yan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"suo1"},
{"chun2"},
{"----"},
{"ai1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"tang2"},
{"zuo4"},
{"----"},
{"----"},
{"wu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ma4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lao2"},
{"----"},
{"----"},
{"----"},
{"huan4"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"feng3"},
{"----"},
{"hu3"},
{"----"},
{"shou4"},
{"wei2"},
{"----"},
{"chang4"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"----"},
{"yo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tuo4"},
{"----"},
{"----"},
{"zhou1"},
{"----"},
{"ken3"},
{"zhuo2"},
{"----"},
{"shang1"},
{"----"},
{"----"},
{"----"},
{"a1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tao2"},
{"dan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chuo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fei1"},
{"----"},
{"----"},
{"pi2"},
{"sha2"},
{"la1"},
{"ze2"},
{"----"},
{"----"},
{"pa1"},
{"----"},
{"se4"},
{"zhuan4"},
{"nie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ding4"},
{"----"},
{"xiao4"},
{"----"},
{"----"},
{"chi4"},
{"ti2"},
{"----"},
{"jiu1"},
{"----"},
{"ka1"},
{"yong2"},
{"wei4"},
{"nan2"},
{"shan4"},
{"----"},
{"----"},
{"la3"},
{"jie1"},
{"hou2"},
{"han3"},
{"die2"},
{"----"},
{"----"},
{"----"},
{"re3"},
{"----"},
{"yin1"},
{"----"},
{"----"},
{"o1"},
{"----"},
{"----"},
{"----"},
{"chuan3"},
{"hui4"},
{"----"},
{"----"},
{"xi3"},
{"he1"},
{"----"},
{"kui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xuan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li2"},
{"----"},
{"cha1"},
{"----"},
{"miao1"},
{"----"},
{"pen1"},
{"----"},
{"kui2"},
{"----"},
{"yu4"},
{"----"},
{"lou2"},
{"ku4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sha4"},
{"xiu4"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ke4"},
{"ta4"},
{"sang3"},
{"chen1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi4"},
{"ge2"},
{"----"},
{"jue1"},
{"----"},
{"weng1"},
{"----"},
{"si4"},
{"chi1"},
{"hao2"},
{"suo1"},
{"----"},
{"hai1"},
{"----"},
{"qin2"},
{"nie4"},
{"he1"},
{"----"},
{"----"},
{"en4"},
{"----"},
{"----"},
{"----"},
{"ai4"},
{"----"},
{"----"},
{"----"},
{"ao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sou4"},
{"sou3"},
{"----"},
{"di2"},
{"qi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cao2"},
{"jia1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ga2"},
{"jia3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xu1"},
{"----"},
{"----"},
{"ma5"},
{"----"},
{"----"},
{"----"},
{"du1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ying1"},
{"----"},
{"----"},
{"mi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zuo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhu3"},
{"chao2"},
{"----"},
{"zui3"},
{"----"},
{"si1"},
{"----"},
{"----"},
{"liao2"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"hei1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ceng1"},
{"----"},
{"ye1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pu1"},
{"jue1"},
{"qin2"},
{"----"},
{"----"},
{"lu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"o1"},
{"----"},
{"jin4"},
{"----"},
{"----"},
{"----"},
{"qi4"},
{"e4"},
{"zao4"},
{"yi1"},
{"shi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ga2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ru2"},
{"hao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hao2"},
{"ti4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nang1"},
{"----"},
{"----"},
{"wei2"},
{"----"},
{"----"},
{"qiu2"},
{"si4"},
{"----"},
{"jian3"},
{"hui2"},
{"xin4"},
{"yin1"},
{"nan1"},
{"tuan2"},
{"----"},
{"dun4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hu2"},
{"----"},
{"yuan2"},
{"----"},
{"----"},
{"kun4"},
{"cong1"},
{"----"},
{"----"},
{"wei2"},
{"lun2"},
{"----"},
{"----"},
{"----"},
{"ling2"},
{"gu4"},
{"----"},
{"----"},
{"guo2"},
{"tu2"},
{"you4"},
{"----"},
{"----"},
{"----"},
{"pu3"},
{"yu3"},
{"----"},
{"yuan2"},
{"----"},
{"quan1"},
{"yu3"},
{"qing1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yuan2"},
{"----"},
{"----"},
{"tu3"},
{"----"},
{"----"},
{"----"},
{"sheng4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zai4"},
{"wei2"},
{"ge1"},
{"----"},
{"wu1"},
{"gui1"},
{"pi3"},
{"yi2"},
{"di4"},
{"----"},
{"----"},
{"zhen4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kuang4"},
{"chang3"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jun1"},
{"----"},
{"----"},
{"fang1"},
{"----"},
{"ben4"},
{"tan1"},
{"kan3"},
{"huai4"},
{"zuo4"},
{"keng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kuai4"},
{"----"},
{"----"},
{"jian1"},
{"tan2"},
{"----"},
{"ba4"},
{"wu4"},
{"fen2"},
{"zhui4"},
{"po1"},
{"----"},
{"----"},
{"kun1"},
{"----"},
{"tan3"},
{"----"},
{"tuo2"},
{"gan1"},
{"ping2"},
{"dian4"},
{"----"},
{"ni2"},
{"----"},
{"pi1"},
{"----"},
{"----"},
{"----"},
{"ao4"},
{"----"},
{"----"},
{"----"},
{"ke3"},
{"----"},
{"----"},
{"----"},
{"chi2"},
{"che4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chui2"},
{"----"},
{"long3"},
{"----"},
{"lu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xing2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lei3"},
{"gai1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"duo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yin2"},
{"----"},
{"gou4"},
{"yuan2"},
{"die2"},
{"----"},
{"ken3"},
{"----"},
{"----"},
{"e4"},
{"----"},
{"dian4"},
{"----"},
{"ya4"},
{"kua3"},
{"----"},
{"----"},
{"----"},
{"kai3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"geng3"},
{"ai1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mai2"},
{"----"},
{"----"},
{"cheng2"},
{"yan2"},
{"----"},
{"----"},
{"lei4"},
{"----"},
{"pu3"},
{"cheng2"},
{"----"},
{"----"},
{"shi2"},
{"xun1"},
{"guo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"bu4"},
{"----"},
{"----"},
{"----"},
{"pi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi2"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"pei2"},
{"ji1"},
{"----"},
{"----"},
{"sao4"},
{"----"},
{"----"},
{"ku1"},
{"----"},
{"tang2"},
{"----"},
{"----"},
{"----"},
{"dui1"},
{"jin3"},
{"----"},
{"----"},
{"----"},
{"peng2"},
{"----"},
{"tu4"},
{"----"},
{"----"},
{"----"},
{"qian4"},
{"----"},
{"----"},
{"----"},
{"duo4"},
{"----"},
{"----"},
{"----"},
{"yin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"die2"},
{"----"},
{"hou4"},
{"bao3"},
{"----"},
{"----"},
{"di1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"du3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ta1"},
{"cheng2"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"----"},
{"ta3"},
{"----"},
{"----"},
{"----"},
{"tang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sai1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ge2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tian2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu2"},
{"----"},
{"chi2"},
{"man4"},
{"----"},
{"jing4"},
{"----"},
{"shu4"},
{"----"},
{"----"},
{"----"},
{"yong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zeng1"},
{"xu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"dun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"yong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"he4"},
{"----"},
{"----"},
{"----"},
{"hao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi4"},
{"ren2"},
{"----"},
{"zhuang4"},
{"----"},
{"sheng1"},
{"----"},
{"----"},
{"qiao4"},
{"----"},
{"----"},
{"hu2"},
{"----"},
{"----"},
{"yi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chu3"},
{"----"},
{"----"},
{"bei4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu4"},
{"----"},
{"xia4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kui2"},
{"xi4"},
{"wai4"},
{"----"},
{"----"},
{"su4"},
{"duo1"},
{"----"},
{"ye4"},
{"----"},
{"----"},
{"gou4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yin2"},
{"huo3"},
{"----"},
{"da4"},
{"----"},
{"tian1"},
{"tai4"},
{"fu1"},
{"----"},
{"yao1"},
{"yang1"},
{"hang1"},
{"----"},
{"shi1"},
{"----"},
{"----"},
{"tou2"},
{"----"},
{"----"},
{"yi2"},
{"kua1"},
{"jia1"},
{"duo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lian2"},
{"huan4"},
{"----"},
{"yan3"},
{"----"},
{"----"},
{"qi2"},
{"nai4"},
{"feng4"},
{"----"},
{"fen4"},
{"----"},
{"----"},
{"kui2"},
{"zou4"},
{"----"},
{"qi4"},
{"----"},
{"----"},
{"ben1"},
{"yi4"},
{"jiang3"},
{"tao4"},
{"zhuang3"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian4"},
{"----"},
{"she1"},
{"----"},
{"----"},
{"ao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nu:3"},
{"nu2"},
{"----"},
{"nai3"},
{"----"},
{"jian1"},
{"ta1"},
{"----"},
{"----"},
{"----"},
{"hao3"},
{"----"},
{"----"},
{"----"},
{"shuo4"},
{"ru2"},
{"fei1"},
{"wang4"},
{"----"},
{"zhuang1"},
{"fu4"},
{"ma1"},
{"----"},
{"ren4"},
{"----"},
{"----"},
{"yan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"du4"},
{"ji4"},
{"----"},
{"----"},
{"yao1"},
{"jin4"},
{"----"},
{"miao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"niu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi3"},
{"yu2"},
{"tuo3"},
{"----"},
{"----"},
{"fang2"},
{"wu3"},
{"yu4"},
{"gui1"},
{"----"},
{"----"},
{"ni1"},
{"zhou2"},
{"----"},
{"----"},
{"da2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mei4"},
{"----"},
{"qi1"},
{"----"},
{"----"},
{"qie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mu3"},
{"----"},
{"----"},
{"----"},
{"zi3"},
{"shi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jie3"},
{"gu1"},
{"si4"},
{"xing4"},
{"wei3"},
{"----"},
{"----"},
{"shan1"},
{"pin1"},
{"----"},
{"yao2"},
{"----"},
{"jiang1"},
{"shu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiao1"},
{"----"},
{"lao3"},
{"----"},
{"----"},
{"yi2"},
{"----"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yin1"},
{"----"},
{"----"},
{"----"},
{"zi1"},
{"----"},
{"wei1"},
{"----"},
{"wa2"},
{"lou2"},
{"ya4"},
{"rao3"},
{"jiao1"},
{"luan2"},
{"ping1"},
{"----"},
{"----"},
{"li3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"suo1"},
{"----"},
{"wei3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"niang2"},
{"----"},
{"----"},
{"----"},
{"nuo2"},
{"----"},
{"----"},
{"juan1"},
{"shen1"},
{"----"},
{"----"},
{"di4"},
{"----"},
{"e2"},
{"----"},
{"----"},
{"----"},
{"wan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"wa1"},
{"----"},
{"xian2"},
{"----"},
{"qu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chang1"},
{"----"},
{"----"},
{"----"},
{"e1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"po2"},
{"----"},
{"----"},
{"wan3"},
{"biao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jie2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jing4"},
{"----"},
{"----"},
{"lan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ying1"},
{"chan2"},
{"shen3"},
{"ting2"},
{"----"},
{"----"},
{"wu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mei2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mei4"},
{"yuan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi4"},
{"xi2"},
{"----"},
{"ying4"},
{"----"},
{"----"},
{"chi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gou4"},
{"----"},
{"----"},
{"jia4"},
{"sao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji2"},
{"----"},
{"----"},
{"xian2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ai4"},
{"----"},
{"pin2"},
{"----"},
{"piao2"},
{"----"},
{"lei2"},
{"----"},
{"----"},
{"----"},
{"zhang1"},
{"----"},
{"----"},
{"----"},
{"li2"},
{"di2"},
{"----"},
{"yan1"},
{"----"},
{"----"},
{"chang2"},
{"----"},
{"----"},
{"nen4"},
{"----"},
{"mo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"shan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"niao3"},
{"----"},
{"ying2"},
{"----"},
{"----"},
{"ma1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shuang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zi3"},
{"jie2"},
{"----"},
{"jue2"},
{"kong3"},
{"yun4"},
{"----"},
{"zi4"},
{"cun2"},
{"sun1"},
{"fu2"},
{"bei4"},
{"zi1"},
{"xiao4"},
{"----"},
{"meng4"},
{"----"},
{"----"},
{"bao1"},
{"ji4"},
{"gu1"},
{"nu2"},
{"xue2"},
{"----"},
{"----"},
{"hai2"},
{"luan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu2"},
{"chan2"},
{"----"},
{"zi1"},
{"----"},
{"fu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ru2"},
{"----"},
{"----"},
{"nie4"},
{"----"},
{"----"},
{"----"},
{"ning2"},
{"----"},
{"ta1"},
{"gui3"},
{"zhai2"},
{"----"},
{"yu3"},
{"shou3"},
{"an1"},
{"----"},
{"song4"},
{"wan2"},
{"----"},
{"----"},
{"hong2"},
{"----"},
{"----"},
{"----"},
{"mi4"},
{"----"},
{"dang4"},
{"----"},
{"zong1"},
{"guan1"},
{"zhou4"},
{"ding4"},
{"wan3"},
{"yi2"},
{"bao3"},
{"shi2"},
{"----"},
{"chong3"},
{"shen3"},
{"ke4"},
{"xuan1"},
{"shi4"},
{"you4"},
{"huan4"},
{"----"},
{"----"},
{"----"},
{"xian4"},
{"gong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zai3"},
{"----"},
{"----"},
{"hai4"},
{"yan4"},
{"xiao1"},
{"jia1"},
{"----"},
{"chen2"},
{"rong2"},
{"----"},
{"----"},
{"----"},
{"kuan1"},
{"bin1"},
{"su4"},
{"----"},
{"----"},
{"ji4"},
{"----"},
{"ji4"},
{"yin2"},
{"mi4"},
{"kou4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu4"},
{"----"},
{"----"},
{"----"},
{"mei4"},
{"----"},
{"han2"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qin3"},
{"mo4"},
{"cha2"},
{"----"},
{"gua3"},
{"----"},
{"----"},
{"wu4"},
{"liao2"},
{"----"},
{"----"},
{"zhai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liao2"},
{"----"},
{"huan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cun4"},
{"dui4"},
{"si4"},
{"xun2"},
{"dao3"},
{"----"},
{"----"},
{"shou4"},
{"----"},
{"feng1"},
{"----"},
{"----"},
{"she4"},
{"----"},
{"jiang1"},
{"----"},
{"----"},
{"wei4"},
{"zun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiao3"},
{"----"},
{"shao4"},
{"----"},
{"----"},
{"er3"},
{"ga3"},
{"jian1"},
{"----"},
{"chen2"},
{"----"},
{"shang4"},
{"----"},
{"----"},
{"chang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wang1"},
{"----"},
{"you2"},
{"----"},
{"----"},
{"yao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ga4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiu4"},
{"----"},
{"----"},
{"gan1"},
{"----"},
{"----"},
{"----"},
{"shi1"},
{"yin3"},
{"chi3"},
{"kao1"},
{"ni2"},
{"jin3"},
{"wei3"},
{"sui1"},
{"ju2"},
{"pi4"},
{"ceng2"},
{"----"},
{"----"},
{"ju1"},
{"----"},
{"----"},
{"qu1"},
{"ti4"},
{"jie4"},
{"wu1"},
{"----"},
{"----"},
{"shi3"},
{"bing3"},
{"ji1"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"zhan3"},
{"----"},
{"----"},
{"----"},
{"e1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu3"},
{"----"},
{"tu2"},
{"lu:3"},
{"----"},
{"xi3"},
{"----"},
{"lu:3"},
{"ju4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"che4"},
{"zhun1"},
{"----"},
{"shan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"qi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu3"},
{"----"},
{"sui4"},
{"qi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji2"},
{"qian1"},
{"----"},
{"----"},
{"qi2"},
{"cen2"},
{"----"},
{"----"},
{"cha4"},
{"----"},
{"qu1"},
{"gang3"},
{"xian4"},
{"----"},
{"lan2"},
{"dao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ke3"},
{"gou3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan2"},
{"----"},
{"xiu4"},
{"jia3"},
{"ling3"},
{"----"},
{"----"},
{"----"},
{"dai4"},
{"----"},
{"yue4"},
{"----"},
{"hu4"},
{"----"},
{"min2"},
{"an4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xun2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xia2"},
{"----"},
{"----"},
{"jiao4"},
{"zheng1"},
{"luan2"},
{"----"},
{"e2"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"qiao4"},
{"----"},
{"----"},
{"feng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jun4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lao2"},
{"lai2"},
{"----"},
{"----"},
{"kong1"},
{"chong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cui1"},
{"----"},
{"ya2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"guo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yao2"},
{"----"},
{"yan1"},
{"song1"},
{"----"},
{"beng1"},
{"----"},
{"----"},
{"----"},
{"zhan3"},
{"gu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wei1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zai3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"sheng4"},
{"mei2"},
{"qian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lou3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"song1"},
{"----"},
{"zi1"},
{"wei2"},
{"----"},
{"----"},
{"cuo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lin2"},
{"----"},
{"----"},
{"----"},
{"deng4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wei1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chuan1"},
{"zhou1"},
{"----"},
{"----"},
{"xun2"},
{"chao2"},
{"----"},
{"----"},
{"gong1"},
{"zuo3"},
{"qiao3"},
{"ju4"},
{"gong3"},
{"----"},
{"wu1"},
{"----"},
{"----"},
{"cha1"},
{"----"},
{"----"},
{"ji3"},
{"yi3"},
{"si4"},
{"ba1"},
{"----"},
{"----"},
{"xiang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xun4"},
{"jin1"},
{"----"},
{"----"},
{"bi4"},
{"shi4"},
{"bu4"},
{"----"},
{"shuai4"},
{"fan1"},
{"----"},
{"shi1"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"wei2"},
{"zhang4"},
{"tang3"},
{"----"},
{"----"},
{"pei4"},
{"pa4"},
{"tie3"},
{"----"},
{"lian2"},
{"zhi4"},
{"zhou3"},
{"bo2"},
{"zhi4"},
{"di4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dai4"},
{"zheng4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi2"},
{"bang1"},
{"----"},
{"----"},
{"dao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wei2"},
{"chang2"},
{"----"},
{"----"},
{"ze2"},
{"guo2"},
{"mao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mi4"},
{"----"},
{"wo4"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"man4"},
{"mu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fan1"},
{"chuang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gan1"},
{"ping2"},
{"nian2"},
{"----"},
{"bing4"},
{"----"},
{"xing4"},
{"----"},
{"----"},
{"huan4"},
{"you4"},
{"you1"},
{"----"},
{"guang3"},
{"pi3"},
{"----"},
{"----"},
{"----"},
{"zhuang1"},
{"----"},
{"qing4"},
{"bi4"},
{"----"},
{"----"},
{"chuang2"},
{"gui3"},
{"----"},
{"----"},
{"----"},
{"xu4"},
{"lu2"},
{"wu3"},
{"----"},
{"ku4"},
{"ying4"},
{"de5"},
{"pao2"},
{"dian4"},
{"----"},
{"miao4"},
{"geng1"},
{"----"},
{"fu3"},
{"----"},
{"pang2"},
{"fei4"},
{"xiang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiu1"},
{"du4"},
{"zuo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ting2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"----"},
{"an1"},
{"shu4"},
{"kang1"},
{"yong1"},
{"tuo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lian2"},
{"lang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jin3"},
{"ao2"},
{"kuo4"},
{"----"},
{"----"},
{"liao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"----"},
{"lin3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan2"},
{"ting2"},
{"----"},
{"----"},
{"jian4"},
{"----"},
{"----"},
{"----"},
{"gong3"},
{"nian4"},
{"kai1"},
{"bian4"},
{"yi4"},
{"qi4"},
{"nong4"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"bi4"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"shi4"},
{"----"},
{"shi4"},
{"----"},
{"gong1"},
{"----"},
{"yin3"},
{"----"},
{"fu2"},
{"hong2"},
{"----"},
{"----"},
{"chi2"},
{"----"},
{"----"},
{"----"},
{"di4"},
{"zhang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mi2"},
{"xian2"},
{"hu2"},
{"----"},
{"nu3"},
{"jing4"},
{"----"},
{"----"},
{"mi3"},
{"----"},
{"wan1"},
{"----"},
{"ruo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dan4"},
{"qiang2"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"gou4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gui1"},
{"dang1"},
{"----"},
{"lu4"},
{"tuan4"},
{"hui4"},
{"zhi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xing2"},
{"----"},
{"tong2"},
{"----"},
{"yan4"},
{"----"},
{"----"},
{"cai3"},
{"biao1"},
{"----"},
{"bin1"},
{"peng2"},
{"----"},
{"----"},
{"zhang1"},
{"ying3"},
{"----"},
{"chi4"},
{"----"},
{"----"},
{"----"},
{"pang2"},
{"----"},
{"yi4"},
{"----"},
{"che4"},
{"bi3"},
{"----"},
{"----"},
{"----"},
{"wang3"},
{"zheng1"},
{"cu2"},
{"----"},
{"jing4"},
{"dai4"},
{"----"},
{"xun4"},
{"hen3"},
{"yang2"},
{"huai2"},
{"lu:4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xu2"},
{"----"},
{"tu2"},
{"----"},
{"----"},
{"lai2"},
{"----"},
{"de2"},
{"pai2"},
{"xi3"},
{"----"},
{"----"},
{"chang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"xun2"},
{"----"},
{"----"},
{"yao2"},
{"wei1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zheng1"},
{"----"},
{"de2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiao4"},
{"hui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xin1"},
{"----"},
{"bi4"},
{"yi4"},
{"----"},
{"----"},
{"dao1"},
{"----"},
{"----"},
{"ji4"},
{"ren3"},
{"----"},
{"chan4"},
{"tan3"},
{"te4"},
{"te4"},
{"----"},
{"----"},
{"----"},
{"cun3"},
{"zhi4"},
{"wang4"},
{"mang2"},
{"----"},
{"----"},
{"----"},
{"tian3"},
{"----"},
{"----"},
{"zhong1"},
{"chong1"},
{"----"},
{"----"},
{"wu3"},
{"----"},
{"----"},
{"you1"},
{"----"},
{"----"},
{"zhong1"},
{"kuai4"},
{"----"},
{"bian4"},
{"zhi4"},
{"----"},
{"----"},
{"chen2"},
{"----"},
{"----"},
{"----"},
{"nian4"},
{"----"},
{"----"},
{"niu3"},
{"----"},
{"----"},
{"xin1"},
{"----"},
{"hu1"},
{"kai4"},
{"fen4"},
{"huai2"},
{"tai4"},
{"song3"},
{"wu3"},
{"ou4"},
{"chang4"},
{"chuang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zuo4"},
{"zen3"},
{"yang4"},
{"----"},
{"----"},
{"nu4"},
{"----"},
{"zheng1"},
{"pa4"},
{"bu4"},
{"----"},
{"----"},
{"hu4"},
{"----"},
{"da2"},
{"lian2"},
{"si1"},
{"----"},
{"----"},
{"dai4"},
{"yi2"},
{"----"},
{"----"},
{"----"},
{"ji2"},
{"peng1"},
{"xing4"},
{"yuan4"},
{"ni2"},
{"guai4"},
{"fei4"},
{"----"},
{"----"},
{"----"},
{"qie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zong3"},
{"dui4"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"ren4"},
{"xun2"},
{"shi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lian4"},
{"----"},
{"huang3"},
{"----"},
{"----"},
{"kong3"},
{"----"},
{"heng2"},
{"----"},
{"----"},
{"shu4"},
{"----"},
{"----"},
{"----"},
{"yang4"},
{"hui4"},
{"----"},
{"----"},
{"jia2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hui1"},
{"zi4"},
{"xu4"},
{"----"},
{"----"},
{"nu:4"},
{"hen4"},
{"en1"},
{"ke4"},
{"dong4"},
{"tian2"},
{"gong1"},
{"----"},
{"xi1"},
{"qia4"},
{"----"},
{"----"},
{"ken3"},
{"----"},
{"----"},
{"e3"},
{"----"},
{"tong4"},
{"yan1"},
{"kai3"},
{"ce4"},
{"nao3"},
{"yun4"},
{"----"},
{"yong3"},
{"----"},
{"----"},
{"----"},
{"kun3"},
{"qiao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"ti4"},
{"han4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"hui3"},
{"----"},
{"bei4"},
{"----"},
{"----"},
{"----"},
{"song3"},
{"quan1"},
{"----"},
{"li3"},
{"----"},
{"wu4"},
{"you1"},
{"----"},
{"----"},
{"huan4"},
{"----"},
{"----"},
{"yue4"},
{"----"},
{"nin2"},
{"----"},
{"----"},
{"que4"},
{"xuan2"},
{"qian1"},
{"----"},
{"min3"},
{"----"},
{"fei3"},
{"bei1"},
{"----"},
{"cui4"},
{"----"},
{"----"},
{"----"},
{"ji4"},
{"----"},
{"----"},
{"xing4"},
{"dao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qing2"},
{"chou2"},
{"----"},
{"----"},
{"----"},
{"jing1"},
{"wan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huo4"},
{"----"},
{"----"},
{"----"},
{"ti4"},
{"----"},
{"----"},
{"wang3"},
{"----"},
{"hu1"},
{"----"},
{"xi1"},
{"tang3"},
{"----"},
{"wei2"},
{"hui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian4"},
{"ju4"},
{"can3"},
{"cheng2"},
{"----"},
{"bei4"},
{"qie4"},
{"can2"},
{"dan4"},
{"guan4"},
{"duo4"},
{"----"},
{"----"},
{"xiang3"},
{"zhui4"},
{"----"},
{"huang2"},
{"----"},
{"----"},
{"re3"},
{"xing1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiao3"},
{"chou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qian1"},
{"----"},
{"yu4"},
{"yu2"},
{"----"},
{"----"},
{"----"},
{"min3"},
{"bi4"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"e4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gan3"},
{"yun4"},
{"----"},
{"----"},
{"leng4"},
{"fen4"},
{"----"},
{"kui4"},
{"kui4"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yuan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ci2"},
{"----"},
{"qie4"},
{"----"},
{"huang1"},
{"----"},
{"shen4"},
{"----"},
{"----"},
{"she4"},
{"----"},
{"----"},
{"----"},
{"mu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"te4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"man4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hui4"},
{"kai3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wei4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yong1"},
{"----"},
{"kang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bie1"},
{"----"},
{"----"},
{"zeng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chong1"},
{"han1"},
{"qi4"},
{"----"},
{"----"},
{"jing3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"han4"},
{"----"},
{"----"},
{"----"},
{"dong3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"----"},
{"ao4"},
{"mao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"men4"},
{"lan3"},
{"----"},
{"lin3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nuo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"meng3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhuang4"},
{"----"},
{"ge1"},
{"----"},
{"wu4"},
{"jian1"},
{"xu1"},
{"shu4"},
{"rong2"},
{"xi4"},
{"cheng2"},
{"wo3"},
{"jie4"},
{"----"},
{"----"},
{"----"},
{"huo4"},
{"qiang1"},
{"zhan4"},
{"----"},
{"qi1"},
{"jia2"},
{"----"},
{"----"},
{"----"},
{"ji3"},
{"----"},
{"kan1"},
{"ji2"},
{"----"},
{"gai4"},
{"deng3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jie2"},
{"----"},
{"jian3"},
{"----"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chuo1"},
{"dai4"},
{"----"},
{"----"},
{"hu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hu4"},
{"li4"},
{"fang2"},
{"suo3"},
{"bian3"},
{"----"},
{"jiong1"},
{"----"},
{"----"},
{"----"},
{"shan1"},
{"hu4"},
{"fei1"},
{"----"},
{"shou3"},
{"----"},
{"cai2"},
{"za1"},
{"----"},
{"----"},
{"pu1"},
{"ba1"},
{"da3"},
{"reng1"},
{"----"},
{"----"},
{"----"},
{"tuo1"},
{"----"},
{"----"},
{"kang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kou4"},
{"----"},
{"----"},
{"qian1"},
{"zhi2"},
{"----"},
{"kuo4"},
{"men2"},
{"sao4"},
{"yang2"},
{"niu3"},
{"ban4"},
{"che3"},
{"rao3"},
{"----"},
{"----"},
{"ban1"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"pi1"},
{"----"},
{"----"},
{"e4"},
{"----"},
{"zhao3"},
{"cheng2"},
{"ji4"},
{"----"},
{"----"},
{"----"},
{"chao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"ba3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"shu1"},
{"zhua1"},
{"----"},
{"tou2"},
{"dou3"},
{"kang4"},
{"zhe2"},
{"----"},
{"fu3"},
{"pao1"},
{"----"},
{"----"},
{"----"},
{"tuan2"},
{"kou1"},
{"lun2"},
{"qiang3"},
{"----"},
{"hu4"},
{"bao4"},
{"----"},
{"----"},
{"peng1"},
{"----"},
{"----"},
{"pi1"},
{"tai2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bao4"},
{"----"},
{"----"},
{"----"},
{"di3"},
{"----"},
{"----"},
{"----"},
{"mo3"},
{"----"},
{"----"},
{"ya1"},
{"chou1"},
{"----"},
{"min3"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"zhu3"},
{"dan1"},
{"chai1"},
{"mu3"},
{"----"},
{"la1"},
{"fu3"},
{"----"},
{"ban4"},
{"pai1"},
{"lin1"},
{"----"},
{"guai3"},
{"----"},
{"ju4"},
{"tuo4"},
{"ba2"},
{"----"},
{"tuo1"},
{"ao4"},
{"ju1"},
{"zhuo2"},
{"pan4"},
{"zhao1"},
{"bai4"},
{"----"},
{"----"},
{"ni3"},
{"----"},
{"----"},
{"long3"},
{"jian3"},
{"----"},
{"yong1"},
{"lan2"},
{"ning4"},
{"bo1"},
{"ze2"},
{"----"},
{"----"},
{"kuo4"},
{"shi4"},
{"jie2"},
{"zheng3"},
{"----"},
{"gong3"},
{"----"},
{"quan2"},
{"shuan1"},
{"----"},
{"zan3"},
{"kao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pin1"},
{"zhuai4"},
{"shi2"},
{"na2"},
{"----"},
{"chi2"},
{"gua4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi3"},
{"qie4"},
{"an4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kua4"},
{"----"},
{"----"},
{"tiao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wa1"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"luan2"},
{"----"},
{"zhua1"},
{"ta4"},
{"xie2"},
{"nao2"},
{"dang3"},
{"jiao3"},
{"zheng4"},
{"ji3"},
{"hui1"},
{"----"},
{"----"},
{"ai2"},
{"----"},
{"nuo2"},
{"cuo4"},
{"----"},
{"----"},
{"----"},
{"zhen4"},
{"----"},
{"----"},
{"suo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"ting3"},
{"----"},
{"----"},
{"wan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wu3"},
{"jun4"},
{"----"},
{"tong3"},
{"kun3"},
{"----"},
{"----"},
{"zhuo1"},
{"----"},
{"luo1"},
{"ba1"},
{"han4"},
{"shao1"},
{"nie1"},
{"juan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lao1"},
{"sun3"},
{"----"},
{"jian3"},
{"huan4"},
{"dao3"},
{"----"},
{"----"},
{"----"},
{"peng3"},
{"----"},
{"lie4"},
{"----"},
{"----"},
{"----"},
{"bai3"},
{"ju4"},
{"----"},
{"----"},
{"ai2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chui2"},
{"jie2"},
{"----"},
{"----"},
{"na4"},
{"nian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xian1"},
{"----"},
{"dian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"duo1"},
{"shou4"},
{"diao4"},
{"pou3"},
{"----"},
{"zhang3"},
{"----"},
{"ji3"},
{"tao1"},
{"qia1"},
{"----"},
{"pai2"},
{"----"},
{"----"},
{"----"},
{"ye1"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lu:e4"},
{"----"},
{"tan4"},
{"che4"},
{"----"},
{"jie1"},
{"----"},
{"kong4"},
{"tui1"},
{"yan3"},
{"cuo4"},
{"----"},
{"----"},
{"----"},
{"qian2"},
{"----"},
{"bai1"},
{"----"},
{"----"},
{"lu3"},
{"guo2"},
{"----"},
{"----"},
{"zhi4"},
{"dan3"},
{"----"},
{"shan3"},
{"----"},
{"guan4"},
{"----"},
{"yuan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"kui2"},
{"----"},
{"----"},
{"rou2"},
{"----"},
{"----"},
{"----"},
{"zou4"},
{"xuan1"},
{"miao2"},
{"ti2"},
{"----"},
{"cha1"},
{"----"},
{"----"},
{"----"},
{"yi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ya4"},
{"wo4"},
{"----"},
{"chuai3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kai1"},
{"jiu1"},
{"----"},
{"----"},
{"jie1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"she2"},
{"----"},
{"yuan2"},
{"----"},
{"ye2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lan3"},
{"----"},
{"qin4"},
{"chan1"},
{"ge1"},
{"lou3"},
{"----"},
{"----"},
{"jiao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhan3"},
{"----"},
{"----"},
{"bo2"},
{"chu4"},
{"----"},
{"----"},
{"cuo1"},
{"sao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sou1"},
{"----"},
{"gao3"},
{"----"},
{"shuo4"},
{"sang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nuo4"},
{"----"},
{"----"},
{"----"},
{"tang2"},
{"----"},
{"ban1"},
{"da1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie2"},
{"----"},
{"----"},
{"cha2"},
{"----"},
{"ge2"},
{"----"},
{"en4"},
{"----"},
{"----"},
{"she4"},
{"shu1"},
{"bai3"},
{"yao2"},
{"bin4"},
{"----"},
{"tan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bing4"},
{"----"},
{"shuai1"},
{"----"},
{"----"},
{"----"},
{"zhai1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"luo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cui1"},
{"----"},
{"mo2"},
{"----"},
{"----"},
{"----"},
{"zhi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mo1"},
{"mo2"},
{"zhe2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liao4"},
{"----"},
{"ying1"},
{"jue1"},
{"----"},
{"pie1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cheng1"},
{"sa3"},
{"----"},
{"----"},
{"si1"},
{"----"},
{"----"},
{"----"},
{"zun3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhuang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"che4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liao2"},
{"----"},
{"----"},
{"qiao4"},
{"bo1"},
{"cuo1"},
{"----"},
{"zhuan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nian3"},
{"----"},
{"xie2"},
{"----"},
{"----"},
{"cuan1"},
{"----"},
{"han4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lei4"},
{"----"},
{"----"},
{"shan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cao1"},
{"qing2"},
{"----"},
{"huan4"},
{"----"},
{"qin2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sou3"},
{"----"},
{"----"},
{"----"},
{"zhuo2"},
{"----"},
{"xing3"},
{"----"},
{"ca1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zuan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"nang3"},
{"zhi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shou1"},
{"----"},
{"you1"},
{"gai3"},
{"----"},
{"gong1"},
{"----"},
{"----"},
{"fang4"},
{"zheng4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gu4"},
{"----"},
{"----"},
{"xiao4"},
{"mi3"},
{"----"},
{"----"},
{"di2"},
{"----"},
{"----"},
{"min3"},
{"----"},
{"jiu4"},
{"----"},
{"----"},
{"----"},
{"chi4"},
{"ao2"},
{"----"},
{"----"},
{"jiao4"},
{"----"},
{"lian3"},
{"----"},
{"bi4"},
{"chang3"},
{"----"},
{"----"},
{"----"},
{"gan3"},
{"san4"},
{"----"},
{"----"},
{"dun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jing4"},
{"----"},
{"----"},
{"----"},
{"shu4"},
{"----"},
{"qiao1"},
{"----"},
{"zheng3"},
{"----"},
{"----"},
{"fu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wen2"},
{"----"},
{"----"},
{"----"},
{"zhai1"},
{"bin1"},
{"----"},
{"----"},
{"----"},
{"fei3"},
{"ban1"},
{"----"},
{"lan2"},
{"----"},
{"----"},
{"----"},
{"dou3"},
{"----"},
{"liao4"},
{"----"},
{"hu2"},
{"xie2"},
{"----"},
{"----"},
{"zhen1"},
{"----"},
{"wo4"},
{"----"},
{"----"},
{"jin1"},
{"chi4"},
{"----"},
{"fu3"},
{"----"},
{"zhan3"},
{"----"},
{"zhuo2"},
{"----"},
{"duan4"},
{"----"},
{"si1"},
{"xin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fang1"},
{"----"},
{"----"},
{"yu1"},
{"shi1"},
{"----"},
{"----"},
{"----"},
{"pang2"},
{"----"},
{"zhan1"},
{"mao4"},
{"lu:3"},
{"pei4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xuan2"},
{"jing1"},
{"----"},
{"ni3"},
{"zu2"},
{"----"},
{"----"},
{"liu2"},
{"----"},
{"----"},
{"----"},
{"yi3"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wu2"},
{"----"},
{"ji4"},
{"----"},
{"----"},
{"ri4"},
{"dan4"},
{"jiu4"},
{"zhi3"},
{"zao3"},
{"----"},
{"----"},
{"xun2"},
{"xu4"},
{"----"},
{"----"},
{"gan4"},
{"han4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi2"},
{"kuang4"},
{"----"},
{"----"},
{"wang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yun2"},
{"----"},
{"ang2"},
{"ze4"},
{"----"},
{"----"},
{"kun1"},
{"----"},
{"----"},
{"----"},
{"hao4"},
{"----"},
{"chang1"},
{"----"},
{"ming2"},
{"hun1"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"xi1"},
{"xin1"},
{"----"},
{"----"},
{"----"},
{"tan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xing1"},
{"ying4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chun1"},
{"----"},
{"mei4"},
{"zuo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhao1"},
{"----"},
{"shi4"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"mao3"},
{"ni4"},
{"chang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhou4"},
{"----"},
{"xian3"},
{"----"},
{"----"},
{"chao2"},
{"----"},
{"huang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jin4"},
{"shang3"},
{"----"},
{"----"},
{"yan4"},
{"----"},
{"----"},
{"shai4"},
{"xiao3"},
{"ye4"},
{"yun4"},
{"hui1"},
{"----"},
{"----"},
{"----"},
{"wan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sheng4"},
{"----"},
{"bu1"},
{"----"},
{"----"},
{"wu4"},
{"----"},
{"hui4"},
{"----"},
{"chen2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pu3"},
{"jing3"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"qing2"},
{"----"},
{"jing1"},
{"gui3"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"----"},
{"----"},
{"liang4"},
{"----"},
{"----"},
{"----"},
{"zan4"},
{"----"},
{"xuan1"},
{"----"},
{"----"},
{"xia2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kui2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nuan3"},
{"an4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ming2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ai4"},
{"ji4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu3"},
{"----"},
{"xun1"},
{"yao4"},
{"pu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"nang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yue1"},
{"----"},
{"qu3"},
{"ye4"},
{"geng4"},
{"----"},
{"----"},
{"he2"},
{"----"},
{"cao2"},
{"----"},
{"----"},
{"man4"},
{"----"},
{"ceng2"},
{"ti4"},
{"zui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yue4"},
{"you3"},
{"ruan3"},
{"peng2"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"qu2"},
{"----"},
{"----"},
{"----"},
{"shuo4"},
{"zhen4"},
{"----"},
{"lang3"},
{"----"},
{"----"},
{"----"},
{"wang4"},
{"----"},
{"chao2"},
{"----"},
{"qi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"meng2"},
{"----"},
{"mu4"},
{"----"},
{"wei4"},
{"mo4"},
{"ben3"},
{"zha2"},
{"----"},
{"shu4"},
{"----"},
{"zhu1"},
{"----"},
{"----"},
{"po4"},
{"duo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"xiu3"},
{"----"},
{"----"},
{"sha1"},
{"----"},
{"za2"},
{"quan2"},
{"----"},
{"----"},
{"gan1"},
{"----"},
{"cha4"},
{"shan1"},
{"----"},
{"----"},
{"wu4"},
{"----"},
{"li3"},
{"xing4"},
{"cai2"},
{"cun1"},
{"----"},
{"shao2"},
{"----"},
{"----"},
{"zhang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"du4"},
{"----"},
{"qi3"},
{"shu4"},
{"gang4"},
{"tiao2"},
{"----"},
{"----"},
{"----"},
{"lai2"},
{"----"},
{"----"},
{"yang2"},
{"----"},
{"miao3"},
{"----"},
{"----"},
{"hang2"},
{"----"},
{"bei1"},
{"jie2"},
{"----"},
{"gao3"},
{"yao3"},
{"----"},
{"chu3"},
{"----"},
{"pa2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhu4"},
{"----"},
{"song1"},
{"ban3"},
{"----"},
{"ji2"},
{"----"},
{"----"},
{"gou4"},
{"----"},
{"----"},
{"pi2"},
{"----"},
{"wang3"},
{"----"},
{"fang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhen3"},
{"----"},
{"lin2"},
{"rui4"},
{"----"},
{"mei2"},
{"----"},
{"guo3"},
{"zhi1"},
{"cong1"},
{"----"},
{"----"},
{"----"},
{"shu1"},
{"zao3"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"cheng2"},
{"----"},
{"qiang1"},
{"feng1"},
{"----"},
{"xiao1"},
{"----"},
{"ku1"},
{"ping2"},
{"----"},
{"----"},
{"zhi3"},
{"----"},
{"xiao1"},
{"jia4"},
{"jia1"},
{"ju3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tuo2"},
{"----"},
{"ling2"},
{"bing3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bo4"},
{"mou3"},
{"gan1"},
{"qi1"},
{"ran3"},
{"rou2"},
{"----"},
{"----"},
{"----"},
{"zhe4"},
{"xia2"},
{"you4"},
{"----"},
{"ju3"},
{"tuo4"},
{"zuo4"},
{"----"},
{"ning2"},
{"----"},
{"di3"},
{"----"},
{"----"},
{"cha2"},
{"----"},
{"----"},
{"----"},
{"jiu4"},
{"----"},
{"----"},
{"jian3"},
{"----"},
{"----"},
{"ke1"},
{"nai4"},
{"zhu4"},
{"----"},
{"liu3"},
{"chai2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cheng1"},
{"----"},
{"shi4"},
{"zhi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zha4"},
{"----"},
{"biao1"},
{"zhan4"},
{"jie2"},
{"long2"},
{"dong4"},
{"lu2"},
{"----"},
{"li4"},
{"lan2"},
{"----"},
{"shu4"},
{"----"},
{"shuan1"},
{"----"},
{"----"},
{"qi1"},
{"li4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gua1"},
{"----"},
{"----"},
{"----"},
{"jiao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xu3"},
{"zhu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kao3"},
{"lao3"},
{"----"},
{"----"},
{"----"},
{"yang4"},
{"he2"},
{"gen1"},
{"----"},
{"----"},
{"ge2"},
{"zai1"},
{"luan2"},
{"----"},
{"jie2"},
{"heng2"},
{"gui4"},
{"tao2"},
{"guang4"},
{"wei2"},
{"kuang4"},
{"----"},
{"an4"},
{"an1"},
{"----"},
{"----"},
{"zhuo1"},
{"----"},
{"zhi4"},
{"----"},
{"tong2"},
{"sang1"},
{"----"},
{"huan2"},
{"ju2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ya1"},
{"rao2"},
{"zhen1"},
{"dang4"},
{"qi1"},
{"qiao2"},
{"hua4"},
{"gui4"},
{"jiang3"},
{"zhuang1"},
{"----"},
{"suo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"tong3"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liang2"},
{"----"},
{"ting3"},
{"----"},
{"mei2"},
{"bang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gu4"},
{"----"},
{"----"},
{"----"},
{"zi3"},
{"----"},
{"----"},
{"----"},
{"geng3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shao1"},
{"----"},
{"----"},
{"----"},
{"meng4"},
{"wu2"},
{"li2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"suo1"},
{"----"},
{"ti1"},
{"xie4"},
{"----"},
{"----"},
{"shu1"},
{"----"},
{"fan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian3"},
{"----"},
{"ling2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mian2"},
{"----"},
{"qi2"},
{"----"},
{"gun4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bang4"},
{"----"},
{"----"},
{"zong1"},
{"----"},
{"----"},
{"ji2"},
{"----"},
{"peng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tang2"},
{"----"},
{"----"},
{"di4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sen1"},
{"----"},
{"chui2"},
{"leng2"},
{"----"},
{"----"},
{"----"},
{"ke1"},
{"----"},
{"----"},
{"----"},
{"zhuo1"},
{"guan1"},
{"----"},
{"fen2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"guo3"},
{"----"},
{"----"},
{"----"},
{"yi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liang2"},
{"----"},
{"zhi2"},
{"zhui1"},
{"----"},
{"ju1"},
{"----"},
{"jiao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"du2"},
{"qian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tuo3"},
{"----"},
{"----"},
{"ye1"},
{"----"},
{"----"},
{"----"},
{"duan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shen4"},
{"----"},
{"----"},
{"----"},
{"chuan2"},
{"----"},
{"chun2"},
{"----"},
{"----"},
{"cha2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie1"},
{"----"},
{"----"},
{"jian4"},
{"----"},
{"----"},
{"chu3"},
{"----"},
{"----"},
{"lian4"},
{"leng2"},
{"----"},
{"nan2"},
{"----"},
{"----"},
{"mei2"},
{"----"},
{"----"},
{"xuan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji2"},
{"----"},
{"----"},
{"chu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kai3"},
{"qiu1"},
{"ying2"},
{"----"},
{"----"},
{"lou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gai4"},
{"----"},
{"lan3"},
{"----"},
{"yu2"},
{"chen4"},
{"lu:2"},
{"ju3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lang2"},
{"rong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhen1"},
{"bang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fei3"},
{"zha4"},
{"----"},
{"----"},
{"sun3"},
{"----"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"cui1"},
{"----"},
{"----"},
{"liu2"},
{"----"},
{"----"},
{"que4"},
{"----"},
{"----"},
{"----"},
{"ta4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shuo4"},
{"----"},
{"chui2"},
{"----"},
{"cha2"},
{"----"},
{"huai2"},
{"----"},
{"----"},
{"----"},
{"gao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian4"},
{"----"},
{"----"},
{"----"},
{"bing1"},
{"zhu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cao2"},
{"----"},
{"jin3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhang1"},
{"----"},
{"mo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"heng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiang2"},
{"----"},
{"ying1"},
{"----"},
{"----"},
{"----"},
{"qiao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zun1"},
{"yue4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tuo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ju2"},
{"cheng2"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiang4"},
{"----"},
{"----"},
{"----"},
{"zhu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lu3"},
{"----"},
{"----"},
{"yuan2"},
{"----"},
{"----"},
{"----"},
{"tan2"},
{"----"},
{"----"},
{"----"},
{"xi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qin2"},
{"----"},
{"yan2"},
{"lei2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qing2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lin3"},
{"----"},
{"----"},
{"meng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qian4"},
{"ci4"},
{"huan1"},
{"xin1"},
{"yu2"},
{"----"},
{"----"},
{"ou1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"yi1"},
{"qi1"},
{"----"},
{"----"},
{"----"},
{"kuan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sha4"},
{"----"},
{"----"},
{"xin1"},
{"xie1"},
{"----"},
{"qian4"},
{"----"},
{"----"},
{"ge1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"she4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi3"},
{"zheng4"},
{"ci3"},
{"bu4"},
{"wu3"},
{"qi2"},
{"----"},
{"----"},
{"wai1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dai3"},
{"----"},
{"si3"},
{"jian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"cu2"},
{"yang1"},
{"tian3"},
{"----"},
{"dai4"},
{"shang1"},
{"----"},
{"xun4"},
{"shu1"},
{"can2"},
{"----"},
{"piao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yun3"},
{"lian4"},
{"----"},
{"----"},
{"zhi2"},
{"----"},
{"----"},
{"----"},
{"dan1"},
{"ji2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bin4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu1"},
{"ou1"},
{"duan4"},
{"----"},
{"yin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian4"},
{"----"},
{"hui3"},
{"gu1"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wu2"},
{"----"},
{"mu3"},
{"----"},
{"mei3"},
{"----"},
{"----"},
{"du2"},
{"yu4"},
{"bi3"},
{"bi4"},
{"bi4"},
{"pi2"},
{"----"},
{"bi4"},
{"----"},
{"mao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hao2"},
{"----"},
{"----"},
{"----"},
{"tan3"},
{"----"},
{"----"},
{"----"},
{"cui4"},
{"----"},
{"san1"},
{"----"},
{"----"},
{"----"},
{"shu1"},
{"----"},
{"----"},
{"----"},
{"jian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chang3"},
{"pu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qu2"},
{"----"},
{"shi4"},
{"di3"},
{"min2"},
{"----"},
{"meng2"},
{"qi4"},
{"pie1"},
{"nai3"},
{"----"},
{"dao1"},
{"xian1"},
{"chuan1"},
{"fen1"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"dong1"},
{"qing1"},
{"----"},
{"yin1"},
{"----"},
{"hai4"},
{"yang3"},
{"an1"},
{"ya4"},
{"ke4"},
{"----"},
{"----"},
{"----"},
{"dan4"},
{"lu:4"},
{"----"},
{"----"},
{"yun1"},
{"----"},
{"shui3"},
{"----"},
{"----"},
{"----"},
{"yong3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cuan1"},
{"----"},
{"----"},
{"ting1"},
{"zhi1"},
{"qiu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hui4"},
{"----"},
{"han4"},
{"cha4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi4"},
{"shan4"},
{"----"},
{"han4"},
{"----"},
{"----"},
{"----"},
{"xun4"},
{"si4"},
{"ru3"},
{"gong3"},
{"jiang1"},
{"chi2"},
{"wu1"},
{"----"},
{"----"},
{"tang1"},
{"----"},
{"----"},
{"----"},
{"mi4"},
{"gu3"},
{"wang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tai4"},
{"----"},
{"ji2"},
{"----"},
{"bian4"},
{"----"},
{"wen4"},
{"----"},
{"----"},
{"xiong1"},
{"----"},
{"----"},
{"----"},
{"qi4"},
{"fen2"},
{"----"},
{"----"},
{"qin4"},
{"yi2"},
{"wo4"},
{"----"},
{"yuan2"},
{"hang4"},
{"----"},
{"chen2"},
{"chen2"},
{"----"},
{"----"},
{"dun4"},
{"----"},
{"----"},
{"qi1"},
{"mu4"},
{"----"},
{"----"},
{"ta4"},
{"mian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sha1"},
{"----"},
{"pei4"},
{"----"},
{"----"},
{"----"},
{"gou1"},
{"----"},
{"mei2"},
{"----"},
{"feng1"},
{"ou4"},
{"li4"},
{"lun2"},
{"cang1"},
{"----"},
{"----"},
{"hu4"},
{"mo4"},
{"----"},
{"shu4"},
{"ju3"},
{"----"},
{"----"},
{"tuo2"},
{"----"},
{"he2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fei4"},
{"you2"},
{"----"},
{"zhi4"},
{"zhao3"},
{"gu1"},
{"zhan1"},
{"yan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"qiu2"},
{"----"},
{"----"},
{"----"},
{"quan2"},
{"bo2"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"le4"},
{"----"},
{"----"},
{"hong2"},
{"gan1"},
{"fa3"},
{"mao3"},
{"si4"},
{"----"},
{"----"},
{"----"},
{"fan4"},
{"----"},
{"----"},
{"ning4"},
{"----"},
{"ling2"},
{"pao4"},
{"bo1"},
{"qi4"},
{"----"},
{"ni2"},
{"----"},
{"----"},
{"zhu4"},
{"----"},
{"lei4"},
{"xuan4"},
{"----"},
{"----"},
{"pan4"},
{"min3"},
{"tai4"},
{"yang1"},
{"----"},
{"yong3"},
{"----"},
{"beng4"},
{"----"},
{"long2"},
{"lu2"},
{"----"},
{"luo4"},
{"xie4"},
{"po1"},
{"ze2"},
{"jing1"},
{"----"},
{"----"},
{"jie2"},
{"----"},
{"----"},
{"hui2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yang2"},
{"lie4"},
{"----"},
{"ji4"},
{"----"},
{"----"},
{"----"},
{"sa3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi3"},
{"----"},
{"zhu1"},
{"jiang4"},
{"luo4"},
{"----"},
{"----"},
{"dong4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jin1"},
{"----"},
{"wei3"},
{"----"},
{"----"},
{"hong2"},
{"----"},
{"----"},
{"----"},
{"tao2"},
{"----"},
{"----"},
{"er3"},
{"zhou1"},
{"----"},
{"----"},
{"xun2"},
{"----"},
{"----"},
{"----"},
{"huan2"},
{"----"},
{"huo2"},
{"wa1"},
{"qia4"},
{"pai4"},
{"----"},
{"----"},
{"liu2"},
{"----"},
{"----"},
{"----"},
{"qian3"},
{"jiang4"},
{"jiao1"},
{"zhen1"},
{"----"},
{"zhuo2"},
{"ce4"},
{"----"},
{"kuai4"},
{"ji4"},
{"liu2"},
{"----"},
{"hun2"},
{"hu3"},
{"nong2"},
{"xun2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhe4"},
{"jun4"},
{"----"},
{"bang1"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"huan4"},
{"----"},
{"----"},
{"pu3"},
{"----"},
{"----"},
{"hao4"},
{"lang4"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"hai3"},
{"jin4"},
{"----"},
{"----"},
{"----"},
{"mei3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tu2"},
{"----"},
{"----"},
{"nie4"},
{"----"},
{"----"},
{"xiao1"},
{"she4"},
{"----"},
{"----"},
{"yong3"},
{"----"},
{"xian2"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"juan1"},
{"cen2"},
{"ti4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tao1"},
{"----"},
{"lao4"},
{"lai2"},
{"lian2"},
{"wei2"},
{"wo1"},
{"----"},
{"huan4"},
{"di2"},
{"----"},
{"run4"},
{"jian4"},
{"zhang4"},
{"se4"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"shuan4"},
{"ya2"},
{"----"},
{"----"},
{"ye4"},
{"----"},
{"----"},
{"han2"},
{"----"},
{"----"},
{"he2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhuo1"},
{"dian4"},
{"----"},
{"----"},
{"----"},
{"zi1"},
{"xi1"},
{"----"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"lin4"},
{"tang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nao4"},
{"----"},
{"tao2"},
{"cong2"},
{"----"},
{"----"},
{"----"},
{"fei2"},
{"song1"},
{"----"},
{"pi4"},
{"dan4"},
{"----"},
{"----"},
{"yu1"},
{"----"},
{"gan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yin2"},
{"cui4"},
{"----"},
{"huai2"},
{"----"},
{"----"},
{"shen1"},
{"----"},
{"chun2"},
{"----"},
{"----"},
{"----"},
{"hun4"},
{"----"},
{"yan1"},
{"----"},
{"tian1"},
{"miao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qing1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yuan1"},
{"----"},
{"lu4"},
{"zi4"},
{"du2"},
{"----"},
{"jian4"},
{"sheng2"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"shen3"},
{"shen4"},
{"----"},
{"----"},
{"zhu3"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"qu2"},
{"du4"},
{"----"},
{"zha1"},
{"bo2"},
{"wo4"},
{"----"},
{"----"},
{"----"},
{"wen1"},
{"----"},
{"xie4"},
{"----"},
{"wei4"},
{"----"},
{"gang3"},
{"----"},
{"----"},
{"xuan4"},
{"----"},
{"ke3"},
{"----"},
{"----"},
{"----"},
{"you2"},
{"----"},
{"miao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pai4"},
{"mei2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tuan1"},
{"mian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pen2"},
{"jian1"},
{"----"},
{"hu2"},
{"----"},
{"xiang1"},
{"----"},
{"----"},
{"zhan4"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiao3"},
{"----"},
{"----"},
{"yin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wan1"},
{"shi1"},
{"----"},
{"----"},
{"----"},
{"kui4"},
{"----"},
{"jian4"},
{"xu4"},
{"----"},
{"----"},
{"gai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tang2"},
{"yuan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ke4"},
{"----"},
{"----"},
{"----"},
{"liu1"},
{"----"},
{"----"},
{"ming2"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"pu3"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"zhen1"},
{"sou1"},
{"----"},
{"xiu4"},
{"----"},
{"rong2"},
{"hun4"},
{"----"},
{"----"},
{"niao4"},
{"----"},
{"----"},
{"ru4"},
{"----"},
{"----"},
{"----"},
{"chu2"},
{"pang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian1"},
{"----"},
{"----"},
{"----"},
{"zi1"},
{"----"},
{"----"},
{"----"},
{"fu3"},
{"----"},
{"hua2"},
{"----"},
{"zi3"},
{"tao1"},
{"teng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gun3"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"yan4"},
{"she4"},
{"man3"},
{"ying2"},
{"----"},
{"lu:4"},
{"lan4"},
{"luan2"},
{"----"},
{"bin1"},
{"tan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"di1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"piao4"},
{"----"},
{"----"},
{"----"},
{"qi1"},
{"----"},
{"----"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lou4"},
{"----"},
{"----"},
{"----"},
{"li2"},
{"yan3"},
{"cao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xuan2"},
{"yi1"},
{"man4"},
{"----"},
{"mang3"},
{"----"},
{"ta4"},
{"----"},
{"shu4"},
{"----"},
{"zhang1"},
{"----"},
{"----"},
{"huan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ying2"},
{"xiao1"},
{"----"},
{"----"},
{"----"},
{"lian4"},
{"----"},
{"wei2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pan1"},
{"----"},
{"----"},
{"----"},
{"qian2"},
{"----"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"----"},
{"----"},
{"lao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tan2"},
{"chao2"},
{"----"},
{"----"},
{"----"},
{"shao4"},
{"----"},
{"zhu1"},
{"----"},
{"----"},
{"----"},
{"shan1"},
{"----"},
{"chan2"},
{"----"},
{"tong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cheng2"},
{"----"},
{"----"},
{"----"},
{"che4"},
{"gan3"},
{"----"},
{"----"},
{"si1"},
{"shu4"},
{"peng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ao4"},
{"----"},
{"----"},
{"chan2"},
{"----"},
{"----"},
{"dan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"lian2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lai4"},
{"bin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hao2"},
{"ru2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pu2"},
{"zhuo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"han4"},
{"ying2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fen4"},
{"----"},
{"----"},
{"----"},
{"yue4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"guan4"},
{"----"},
{"----"},
{"hao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ba4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huo3"},
{"----"},
{"mie4"},
{"----"},
{"deng1"},
{"hui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ling2"},
{"zao4"},
{"----"},
{"jiu3"},
{"----"},
{"----"},
{"----"},
{"zhuo2"},
{"----"},
{"zai1"},
{"can4"},
{"yang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gui4"},
{"----"},
{"----"},
{"----"},
{"lu2"},
{"chui1"},
{"----"},
{"----"},
{"----"},
{"yan2"},
{"----"},
{"----"},
{"----"},
{"chao3"},
{"----"},
{"que1"},
{"kang4"},
{"dun4"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"----"},
{"wei3"},
{"qiang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xuan4"},
{"ju4"},
{"tan4"},
{"pao4"},
{"jiong3"},
{"----"},
{"tai2"},
{"----"},
{"bing3"},
{"----"},
{"----"},
{"----"},
{"zhu4"},
{"zha2"},
{"dian3"},
{"----"},
{"----"},
{"lian4"},
{"chi4"},
{"----"},
{"----"},
{"----"},
{"shuo4"},
{"lan4"},
{"ting1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lie4"},
{"----"},
{"yang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hong1"},
{"lao4"},
{"----"},
{"zhu2"},
{"----"},
{"----"},
{"----"},
{"yan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kao3"},
{"----"},
{"fan2"},
{"shao1"},
{"ye4"},
{"hui4"},
{"----"},
{"tang4"},
{"jin4"},
{"re4"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wan2"},
{"----"},
{"peng1"},
{"----"},
{"----"},
{"----"},
{"feng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan1"},
{"han4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huan4"},
{"men4"},
{"----"},
{"dao4"},
{"bei4"},
{"fen2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan4"},
{"yan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ran2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"jian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"sha1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mei2"},
{"----"},
{"----"},
{"zhao4"},
{"wei1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhu3"},
{"----"},
{"----"},
{"----"},
{"bao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu2"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yun4"},
{"----"},
{"----"},
{"----"},
{"ao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"man4"},
{"----"},
{"shang1"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ran2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fan2"},
{"yan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zao4"},
{"----"},
{"sui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cuan4"},
{"----"},
{"zhua3"},
{"----"},
{"pa2"},
{"----"},
{"----"},
{"----"},
{"yuan2"},
{"ai4"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"fu4"},
{"ye2"},
{"ba4"},
{"die1"},
{"----"},
{"yao2"},
{"----"},
{"shuang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pian4"},
{"ban3"},
{"----"},
{"----"},
{"----"},
{"pai2"},
{"du2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"die2"},
{"----"},
{"----"},
{"----"},
{"you3"},
{"----"},
{"----"},
{"ya2"},
{"----"},
{"niu2"},
{"----"},
{"pin4"},
{"----"},
{"mou2"},
{"----"},
{"mu3"},
{"lao2"},
{"----"},
{"----"},
{"----"},
{"mao2"},
{"mu4"},
{"----"},
{"wu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian4"},
{"gu3"},
{"----"},
{"----"},
{"sheng1"},
{"----"},
{"----"},
{"qian1"},
{"----"},
{"----"},
{"----"},
{"te4"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gu4"},
{"xi1"},
{"li2"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"du2"},
{"----"},
{"----"},
{"jian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"quan3"},
{"----"},
{"----"},
{"fan4"},
{"qiu2"},
{"----"},
{"----"},
{"----"},
{"an4"},
{"----"},
{"zhuang4"},
{"guang3"},
{"ma3"},
{"you2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yun3"},
{"kuang2"},
{"niu3"},
{"di2"},
{"----"},
{"----"},
{"----"},
{"bei4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xia2"},
{"----"},
{"hu2"},
{"----"},
{"fei4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gou3"},
{"----"},
{"ju1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ning2"},
{"----"},
{"hen3"},
{"jiao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rong2"},
{"shou4"},
{"----"},
{"----"},
{"du2"},
{"xia2"},
{"shi1"},
{"kuai4"},
{"zheng1"},
{"yu4"},
{"sun1"},
{"yu2"},
{"bi4"},
{"----"},
{"----"},
{"juan4"},
{"li2"},
{"----"},
{"yin2"},
{"suan1"},
{"lang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"xian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ni2"},
{"----"},
{"----"},
{"----"},
{"lie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mi2"},
{"chang1"},
{"yi1"},
{"----"},
{"----"},
{"----"},
{"meng3"},
{"cai1"},
{"cu4"},
{"----"},
{"----"},
{"----"},
{"luo2"},
{"hu2"},
{"----"},
{"----"},
{"wei3"},
{"----"},
{"----"},
{"----"},
{"xing1"},
{"zhu1"},
{"mao1"},
{"wei4"},
{"----"},
{"xian4"},
{"----"},
{"----"},
{"nao2"},
{"----"},
{"----"},
{"hou2"},
{"----"},
{"----"},
{"you2"},
{"----"},
{"cha2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hua2"},
{"yuan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jing4"},
{"----"},
{"----"},
{"zhang1"},
{"----"},
{"ao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"ta3"},
{"----"},
{"xun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xuan2"},
{"----"},
{"----"},
{"shuai4"},
{"----"},
{"yu4"},
{"----"},
{"wang2"},
{"----"},
{"----"},
{"ding1"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ma3"},
{"----"},
{"----"},
{"----"},
{"wen2"},
{"----"},
{"----"},
{"fen1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wan4"},
{"----"},
{"mei2"},
{"----"},
{"----"},
{"wei3"},
{"huan2"},
{"xian4"},
{"----"},
{"ling2"},
{"dai4"},
{"----"},
{"----"},
{"----"},
{"dian4"},
{"----"},
{"----"},
{"xi3"},
{"bo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"po4"},
{"----"},
{"ke1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jia1"},
{"----"},
{"shan1"},
{"----"},
{"----"},
{"zhen1"},
{"----"},
{"jue2"},
{"fa4"},
{"long2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gong3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"luo4"},
{"----"},
{"zhu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"er3"},
{"----"},
{"yao2"},
{"----"},
{"heng2"},
{"----"},
{"----"},
{"----"},
{"ban1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hun2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiu2"},
{"----"},
{"lang2"},
{"li3"},
{"----"},
{"----"},
{"liu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"suo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ju1"},
{"chen1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhuo2"},
{"----"},
{"----"},
{"hu3"},
{"qi2"},
{"----"},
{"kun1"},
{"----"},
{"qi2"},
{"----"},
{"wan3"},
{"----"},
{"cong2"},
{"----"},
{"yan3"},
{"----"},
{"----"},
{"lin2"},
{"qin2"},
{"pi2"},
{"pa2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xia2"},
{"----"},
{"yuan4"},
{"----"},
{"nao3"},
{"hu2"},
{"ying1"},
{"yu2"},
{"----"},
{"rui4"},
{"se4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tang2"},
{"----"},
{"----"},
{"gui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yao2"},
{"ai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jin3"},
{"----"},
{"cui3"},
{"cong1"},
{"----"},
{"li2"},
{"----"},
{"----"},
{"----"},
{"xuan2"},
{"----"},
{"----"},
{"----"},
{"zhang1"},
{"----"},
{"----"},
{"ying1"},
{"----"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"pu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"can4"},
{"qu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wen4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gua1"},
{"----"},
{"die2"},
{"----"},
{"hu4"},
{"----"},
{"piao2"},
{"ban4"},
{"rang2"},
{"----"},
{"wa3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"weng4"},
{"ou1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ling2"},
{"----"},
{"ping2"},
{"ci2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhen1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"meng2"},
{"----"},
{"----"},
{"----"},
{"zeng4"},
{"----"},
{"pi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gan1"},
{"----"},
{"shen4"},
{"----"},
{"tian2"},
{"----"},
{"----"},
{"sheng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sheng1"},
{"----"},
{"----"},
{"yong4"},
{"shuai3"},
{"----"},
{"fu3"},
{"yong3"},
{"beng2"},
{"----"},
{"----"},
{"tian2"},
{"you2"},
{"jia3"},
{"shen1"},
{"----"},
{"dian4"},
{"----"},
{"nan2"},
{"dian4"},
{"----"},
{"ting3"},
{"hua4"},
{"----"},
{"----"},
{"zai1"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chang4"},
{"----"},
{"----"},
{"fan4"},
{"----"},
{"----"},
{"tian2"},
{"jie4"},
{"----"},
{"quan3"},
{"wei4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liu2"},
{"ben3"},
{"zhen3"},
{"xu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lu:e4"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"fan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"chou2"},
{"----"},
{"----"},
{"----"},
{"ji1"},
{"wan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi3"},
{"----"},
{"----"},
{"----"},
{"shu1"},
{"----"},
{"yi2"},
{"ni4"},
{"----"},
{"ding1"},
{"----"},
{"jie2"},
{"liao2"},
{"----"},
{"ge1"},
{"jiu4"},
{"----"},
{"----"},
{"shan4"},
{"----"},
{"yao4"},
{"li4"},
{"yang2"},
{"----"},
{"you2"},
{"ba1"},
{"jie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"chuang1"},
{"feng1"},
{"----"},
{"pao4"},
{"pi2"},
{"gan1"},
{"----"},
{"ci1"},
{"----"},
{"----"},
{"dan3"},
{"zhen3"},
{"----"},
{"----"},
{"teng2"},
{"ju1"},
{"ji2"},
{"----"},
{"----"},
{"----"},
{"jia1"},
{"----"},
{"zha4"},
{"bing4"},
{"----"},
{"zheng4"},
{"yong1"},
{"jing4"},
{"quan2"},
{"----"},
{"----"},
{"yi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yang3"},
{"----"},
{"zhi4"},
{"hen2"},
{"----"},
{"----"},
{"dou4"},
{"----"},
{"----"},
{"tong4"},
{"----"},
{"----"},
{"pi3"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"zhi4"},
{"cuo2"},
{"----"},
{"wu4"},
{"sha1"},
{"lao2"},
{"----"},
{"huan4"},
{"xian2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tan2"},
{"fei4"},
{"----"},
{"----"},
{"chi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"gu4"},
{"----"},
{"----"},
{"wei3"},
{"yu1"},
{"cui4"},
{"----"},
{"zhu2"},
{"----"},
{"dan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hou2"},
{"----"},
{"la4"},
{"----"},
{"----"},
{"----"},
{"yu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jia3"},
{"----"},
{"yi4"},
{"----"},
{"sao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wen1"},
{"ji2"},
{"----"},
{"ban1"},
{"----"},
{"liu2"},
{"cuo2"},
{"shou4"},
{"----"},
{"----"},
{"da2"},
{"bie3"},
{"tan1"},
{"----"},
{"biao1"},
{"----"},
{"----"},
{"luo3"},
{"----"},
{"----"},
{"chou1"},
{"zhang4"},
{"zhai4"},
{"----"},
{"----"},
{"que2"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"----"},
{"yin3"},
{"ying3"},
{"----"},
{"----"},
{"----"},
{"long2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian4"},
{"----"},
{"lai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xuan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gui3"},
{"----"},
{"----"},
{"deng1"},
{"----"},
{"bai2"},
{"bai3"},
{"----"},
{"----"},
{"----"},
{"zao4"},
{"----"},
{"de5"},
{"----"},
{"jie1"},
{"huang2"},
{"gui1"},
{"----"},
{"----"},
{"gao1"},
{"----"},
{"----"},
{"jiao3"},
{"----"},
{"----"},
{"ai2"},
{"----"},
{"hao4"},
{"----"},
{"----"},
{"wan3"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"po2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi2"},
{"----"},
{"----"},
{"zhou4"},
{"jun1"},
{"----"},
{"cun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"min3"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"zhong1"},
{"pen2"},
{"----"},
{"ying2"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"he2"},
{"ang4"},
{"zhan3"},
{"yan2"},
{"jian4"},
{"he2"},
{"----"},
{"kui1"},
{"----"},
{"gai4"},
{"dao4"},
{"pan2"},
{"----"},
{"----"},
{"sheng4"},
{"----"},
{"----"},
{"----"},
{"meng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"guan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mu4"},
{"ding1"},
{"----"},
{"xu1"},
{"mang2"},
{"----"},
{"zhi2"},
{"----"},
{"----"},
{"----"},
{"xiang1"},
{"dun3"},
{"----"},
{"----"},
{"pan4"},
{"----"},
{"dun4"},
{"----"},
{"----"},
{"xing3"},
{"----"},
{"----"},
{"mian3"},
{"----"},
{"----"},
{"miao3"},
{"dan1"},
{"mei2"},
{"----"},
{"kan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi2"},
{"sheng3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhen1"},
{"mian2"},
{"----"},
{"yuan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zha3"},
{"xuan4"},
{"----"},
{"----"},
{"----"},
{"sui1"},
{"----"},
{"mi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chi1"},
{"kuang4"},
{"juan4"},
{"mou2"},
{"----"},
{"tiao4"},
{"----"},
{"yan3"},
{"----"},
{"----"},
{"----"},
{"zhe5"},
{"zheng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"di4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lai4"},
{"jian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jing1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shui4"},
{"sui1"},
{"du1"},
{"----"},
{"bi4"},
{"mu4"},
{"----"},
{"ni4"},
{"----"},
{"----"},
{"jie2"},
{"cai3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"du3"},
{"----"},
{"----"},
{"----"},
{"kui2"},
{"gao1"},
{"rui4"},
{"mao4"},
{"----"},
{"----"},
{"----"},
{"miao2"},
{"chou3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ke4"},
{"sou3"},
{"xia1"},
{"----"},
{"----"},
{"ming2"},
{"man2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"piao3"},
{"cheng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pie1"},
{"----"},
{"qiao2"},
{"----"},
{"zhu3"},
{"deng4"},
{"----"},
{"shun4"},
{"----"},
{"----"},
{"----"},
{"kan4"},
{"----"},
{"----"},
{"tong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhan1"},
{"----"},
{"gu3"},
{"----"},
{"ju4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chu4"},
{"----"},
{"----"},
{"----"},
{"mao2"},
{"jin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi3"},
{"yi3"},
{"----"},
{"zhi1"},
{"----"},
{"shen3"},
{"----"},
{"ju3"},
{"----"},
{"jiao3"},
{"cuo2"},
{"duan3"},
{"ai3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi2"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"fan2"},
{"kuang4"},
{"dang4"},
{"ma3"},
{"sha1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi4"},
{"kan3"},
{"----"},
{"----"},
{"----"},
{"ya4"},
{"pi1"},
{"----"},
{"yan2"},
{"----"},
{"zhuan1"},
{"che1"},
{"----"},
{"----"},
{"yan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zha3"},
{"----"},
{"----"},
{"----"},
{"tuo2"},
{"----"},
{"di3"},
{"zhai4"},
{"zhen1"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"bian1"},
{"----"},
{"----"},
{"peng1"},
{"----"},
{"----"},
{"----"},
{"po4"},
{"----"},
{"----"},
{"shen1"},
{"za2"},
{"----"},
{"li4"},
{"long2"},
{"tong2"},
{"----"},
{"li4"},
{"----"},
{"chu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xing2"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"shuo4"},
{"xia2"},
{"qiao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liu2"},
{"ying4"},
{"mang2"},
{"que4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"peng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ding4"},
{"----"},
{"diao1"},
{"----"},
{"----"},
{"lu4"},
{"ai4"},
{"sui4"},
{"----"},
{"----"},
{"bei1"},
{"----"},
{"dui4"},
{"----"},
{"----"},
{"----"},
{"wan3"},
{"dian3"},
{"----"},
{"bei4"},
{"qi4"},
{"----"},
{"----"},
{"----"},
{"die2"},
{"----"},
{"du2"},
{"----"},
{"jie2"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"peng4"},
{"jian3"},
{"di4"},
{"tan4"},
{"cha2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nian3"},
{"----"},
{"----"},
{"ci2"},
{"----"},
{"----"},
{"----"},
{"bang4"},
{"----"},
{"----"},
{"----"},
{"sang3"},
{"lei3"},
{"cuo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pan2"},
{"----"},
{"----"},
{"----"},
{"zhe2"},
{"ke1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"----"},
{"----"},
{"----"},
{"qing4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qu2"},
{"----"},
{"deng4"},
{"----"},
{"----"},
{"lin2"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiao1"},
{"----"},
{"----"},
{"----"},
{"dun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"meng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi4"},
{"----"},
{"li3"},
{"----"},
{"she4"},
{"----"},
{"si4"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xian1"},
{"----"},
{"qi2"},
{"zhi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"zu3"},
{"zhi1"},
{"----"},
{"----"},
{"zuo4"},
{"qu1"},
{"hu4"},
{"zhu4"},
{"shen2"},
{"sui4"},
{"ci2"},
{"----"},
{"mi2"},
{"----"},
{"----"},
{"xiang2"},
{"----"},
{"tiao1"},
{"piao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji4"},
{"----"},
{"zhen1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dao3"},
{"huo4"},
{"----"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bing3"},
{"jin4"},
{"----"},
{"----"},
{"lu4"},
{"shan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhuo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu3"},
{"yu2"},
{"li2"},
{"----"},
{"qin2"},
{"he2"},
{"----"},
{"xiu4"},
{"si1"},
{"----"},
{"tu1"},
{"----"},
{"----"},
{"gan3"},
{"----"},
{"----"},
{"bing3"},
{"----"},
{"qiu1"},
{"----"},
{"zhong4"},
{"----"},
{"----"},
{"----"},
{"ke1"},
{"miao3"},
{"----"},
{"----"},
{"bi3"},
{"----"},
{"----"},
{"mi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zu1"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"cheng4"},
{"----"},
{"qin2"},
{"yang1"},
{"----"},
{"zhi4"},
{"----"},
{"shu2"},
{"----"},
{"zi3"},
{"----"},
{"ji1"},
{"cheng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi2"},
{"----"},
{"hui4"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"lang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cheng2"},
{"----"},
{"shao1"},
{"shui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ren3"},
{"----"},
{"----"},
{"bai4"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"----"},
{"----"},
{"ke1"},
{"----"},
{"chou2"},
{"----"},
{"----"},
{"su1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wen3"},
{"----"},
{"----"},
{"----"},
{"ji4"},
{"----"},
{"zhen3"},
{"----"},
{"dao4"},
{"jia4"},
{"ji1"},
{"----"},
{"gao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"se4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rang2"},
{"----"},
{"----"},
{"----"},
{"xue2"},
{"----"},
{"jiu1"},
{"qiong2"},
{"----"},
{"qiong2"},
{"kong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chuan1"},
{"zhun1"},
{"tu1"},
{"----"},
{"qie4"},
{"zhai3"},
{"----"},
{"bian3"},
{"----"},
{"yao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiao4"},
{"----"},
{"----"},
{"----"},
{"yao2"},
{"zhi4"},
{"----"},
{"----"},
{"tiao3"},
{"jiao4"},
{"chuang1"},
{"jiong3"},
{"----"},
{"----"},
{"----"},
{"cuan4"},
{"wo1"},
{"----"},
{"ku1"},
{"ke1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kui1"},
{"dou4"},
{"----"},
{"yin4"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"ju4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"long2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu4"},
{"----"},
{"----"},
{"zhan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jing4"},
{"jing4"},
{"zhang1"},
{"----"},
{"----"},
{"jun4"},
{"----"},
{"tong2"},
{"song3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jie2"},
{"----"},
{"duan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhu2"},
{"zhu2"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"gan1"},
{"----"},
{"----"},
{"----"},
{"du3"},
{"ji1"},
{"----"},
{"ba1"},
{"----"},
{"ji2"},
{"----"},
{"zhao4"},
{"sun3"},
{"----"},
{"----"},
{"----"},
{"hu4"},
{"----"},
{"xiao4"},
{"----"},
{"----"},
{"bi3"},
{"jian3"},
{"----"},
{"----"},
{"----"},
{"sheng1"},
{"----"},
{"di2"},
{"----"},
{"----"},
{"chi1"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"----"},
{"tiao2"},
{"si4"},
{"fu2"},
{"----"},
{"ben4"},
{"----"},
{"da2"},
{"zi3"},
{"di4"},
{"----"},
{"ze2"},
{"----"},
{"----"},
{"gou3"},
{"----"},
{"jia1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"po3"},
{"----"},
{"jian1"},
{"----"},
{"long2"},
{"----"},
{"bian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xian3"},
{"----"},
{"qiong2"},
{"----"},
{"deng3"},
{"----"},
{"jin1"},
{"quan2"},
{"----"},
{"----"},
{"fa2"},
{"kuang1"},
{"zhu4"},
{"tong3"},
{"----"},
{"da2"},
{"----"},
{"ce4"},
{"----"},
{"kou4"},
{"----"},
{"bi4"},
{"shai1"},
{"----"},
{"zheng1"},
{"----"},
{"----"},
{"yun2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi4"},
{"----"},
{"----"},
{"xiao3"},
{"shao1"},
{"----"},
{"----"},
{"yan2"},
{"----"},
{"kuai4"},
{"----"},
{"chou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qian1"},
{"----"},
{"jian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bo2"},
{"ji1"},
{"----"},
{"suan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kong1"},
{"qian2"},
{"----"},
{"----"},
{"----"},
{"guan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qie4"},
{"tuo4"},
{"luo2"},
{"dan1"},
{"xiao1"},
{"ruo4"},
{"jian4"},
{"----"},
{"----"},
{"----"},
{"xiang1"},
{"----"},
{"----"},
{"zhen1"},
{"----"},
{"----"},
{"----"},
{"zhu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhuan4"},
{"pian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kui4"},
{"----"},
{"lou3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gao1"},
{"fei3"},
{"----"},
{"----"},
{"gou1"},
{"----"},
{"----"},
{"----"},
{"cuan4"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"chi2"},
{"----"},
{"----"},
{"----"},
{"lan2"},
{"----"},
{"----"},
{"li2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"peng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cu4"},
{"----"},
{"----"},
{"----"},
{"gui3"},
{"su4"},
{"----"},
{"----"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"duan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"deng1"},
{"huang2"},
{"----"},
{"----"},
{"zan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bu4"},
{"zhou4"},
{"lai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mi3"},
{"di2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lei4"},
{"----"},
{"zi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fen3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ba1"},
{"li4"},
{"----"},
{"----"},
{"po4"},
{"----"},
{"cu1"},
{"zhan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"su4"},
{"----"},
{"----"},
{"zi1"},
{"----"},
{"yue4"},
{"zhou1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fen4"},
{"----"},
{"----"},
{"----"},
{"liang2"},
{"----"},
{"----"},
{"liang2"},
{"can4"},
{"jing1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cui4"},
{"----"},
{"----"},
{"lin2"},
{"zong4"},
{"jing1"},
{"----"},
{"----"},
{"san3"},
{"----"},
{"----"},
{"----"},
{"rou2"},
{"----"},
{"----"},
{"xu3"},
{"----"},
{"hu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gao1"},
{"tang2"},
{"qiu3"},
{"----"},
{"cao1"},
{"----"},
{"----"},
{"mi2"},
{"----"},
{"----"},
{"zao1"},
{"kang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nuo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mi4"},
{"----"},
{"----"},
{"xi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wen3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"suo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jin3"},
{"----"},
{"----"},
{"----"},
{"zi3"},
{"----"},
{"----"},
{"----"},
{"lei3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"you2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zuan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiu1"},
{"yu1"},
{"hong2"},
{"zhou4"},
{"qian4"},
{"he2"},
{"yue1"},
{"ji2"},
{"wan2"},
{"kuang4"},
{"ji4"},
{"ren4"},
{"wei3"},
{"yun2"},
{"----"},
{"chun2"},
{"pi1"},
{"sha1"},
{"gang1"},
{"na4"},
{"----"},
{"zong4"},
{"lun2"},
{"fen1"},
{"zhi3"},
{"wen2"},
{"fang3"},
{"----"},
{"----"},
{"niu3"},
{"shu1"},
{"xian4"},
{"gan4"},
{"xie4"},
{"fu2"},
{"lian4"},
{"zu3"},
{"shen1"},
{"xi4"},
{"zhi1"},
{"zhong1"},
{"zhou4"},
{"ban4"},
{"fu2"},
{"chu4"},
{"shao4"},
{"yi4"},
{"jing1"},
{"----"},
{"bang3"},
{"rong2"},
{"jie2"},
{"----"},
{"rao4"},
{"----"},
{"----"},
{"hui4"},
{"gei3"},
{"xuan4"},
{"jiang4"},
{"luo4"},
{"jue2"},
{"jiao3"},
{"tong3"},
{"geng3"},
{"xiao1"},
{"juan4"},
{"xiu4"},
{"----"},
{"sui2"},
{"tao1"},
{"ji4"},
{"ti2"},
{"ji1"},
{"xu4"},
{"ling2"},
{"----"},
{"xu4"},
{"qi3"},
{"fei1"},
{"chuo4"},
{"----"},
{"gun3"},
{"sheng2"},
{"wei2"},
{"mian2"},
{"shou4"},
{"beng1"},
{"chou2"},
{"----"},
{"liu3"},
{"quan3"},
{"zong1"},
{"zhan4"},
{"wan3"},
{"lu:4"},
{"zhui4"},
{"zi1"},
{"ke4"},
{"xiang1"},
{"jian1"},
{"mian3"},
{"lan4"},
{"ti2"},
{"miao3"},
{"qi1"},
{"----"},
{"hui4"},
{"si1"},
{"----"},
{"duan4"},
{"bian4"},
{"----"},
{"gou1"},
{"zhui4"},
{"huan3"},
{"di4"},
{"lu:3"},
{"bian1"},
{"min2"},
{"yuan2"},
{"jin4"},
{"fu2"},
{"ru4"},
{"zhen3"},
{"feng4"},
{"----"},
{"gao3"},
{"chan2"},
{"li2"},
{"yi4"},
{"jian1"},
{"bin1"},
{"piao3"},
{"man4"},
{"lei2"},
{"ying1"},
{"suo1"},
{"mou2"},
{"sao1"},
{"xie2"},
{"liao2"},
{"shan4"},
{"zeng4"},
{"----"},
{"qian3"},
{"sao1"},
{"huan2"},
{"jiao3"},
{"zuan3"},
{"fou3"},
{"----"},
{"gang1"},
{"----"},
{"que1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ying1"},
{"----"},
{"qing4"},
{"xia4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"guan4"},
{"wang3"},
{"----"},
{"----"},
{"wang3"},
{"han3"},
{"----"},
{"luo1"},
{"fu2"},
{"----"},
{"fa2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gu3"},
{"----"},
{"gang1"},
{"ba4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan3"},
{"zhao4"},
{"zui4"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"----"},
{"----"},
{"shu3"},
{"----"},
{"pi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zeng1"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yang2"},
{"----"},
{"qiang1"},
{"----"},
{"mei3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ling2"},
{"----"},
{"----"},
{"di1"},
{"xiu1"},
{"qiang3"},
{"----"},
{"xian4"},
{"----"},
{"----"},
{"qun2"},
{"----"},
{"----"},
{"suo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jie2"},
{"tang1"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lei2"},
{"geng1"},
{"----"},
{"----"},
{"chan4"},
{"yu3"},
{"----"},
{"yi4"},
{"----"},
{"weng1"},
{"----"},
{"----"},
{"----"},
{"chi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"yi4"},
{"----"},
{"ling2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiang2"},
{"xi4"},
{"----"},
{"----"},
{"qiao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"di2"},
{"cui4"},
{"fei3"},
{"----"},
{"----"},
{"----"},
{"zhu4"},
{"jian3"},
{"----"},
{"----"},
{"pian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"he2"},
{"----"},
{"han4"},
{"ao2"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fan1"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"yao4"},
{"lao3"},
{"----"},
{"kao3"},
{"mao4"},
{"zhe3"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"die4"},
{"er2"},
{"shua3"},
{"----"},
{"----"},
{"nai4"},
{"----"},
{"lei3"},
{"----"},
{"zi3"},
{"geng1"},
{"----"},
{"hao4"},
{"yun2"},
{"pa2"},
{"----"},
{"----"},
{"si4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ou3"},
{"lou2"},
{"nou4"},
{"jiang3"},
{"pang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"er3"},
{"----"},
{"----"},
{"ye1"},
{"----"},
{"song3"},
{"----"},
{"----"},
{"chi3"},
{"----"},
{"dan1"},
{"----"},
{"geng3"},
{"----"},
{"----"},
{"nie4"},
{"dan1"},
{"----"},
{"----"},
{"ling2"},
{"----"},
{"----"},
{"----"},
{"liao2"},
{"long2"},
{"zhi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lian2"},
{"----"},
{"----"},
{"----"},
{"pin4"},
{"----"},
{"ju4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kui4"},
{"cong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"yi4"},
{"----"},
{"si4"},
{"zhao4"},
{"----"},
{"rou4"},
{"----"},
{"lei4"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huang1"},
{"----"},
{"----"},
{"xiao4"},
{"----"},
{"zhou3"},
{"----"},
{"du4"},
{"gang1"},
{"rong2"},
{"gan1"},
{"----"},
{"wo4"},
{"chang2"},
{"gu3"},
{"zhi1"},
{"----"},
{"fu1"},
{"fei2"},
{"----"},
{"----"},
{"----"},
{"jian1"},
{"fang2"},
{"zhun1"},
{"----"},
{"na4"},
{"ang1"},
{"ken3"},
{"----"},
{"gong1"},
{"yu4"},
{"----"},
{"yao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fei4"},
{"----"},
{"----"},
{"tai4"},
{"shen4"},
{"zhong3"},
{"zhang4"},
{"xie2"},
{"shen4"},
{"wei4"},
{"zhou4"},
{"----"},
{"dan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bei1"},
{"gua1"},
{"tai1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pang4"},
{"zhen1"},
{"----"},
{"zuo4"},
{"pei1"},
{"jia3"},
{"sheng4"},
{"zhi1"},
{"bao1"},
{"----"},
{"----"},
{"hu2"},
{"----"},
{"----"},
{"yin4"},
{"xu1"},
{"----"},
{"long2"},
{"----"},
{"----"},
{"lu2"},
{"jing4"},
{"----"},
{"yan1"},
{"----"},
{"kua4"},
{"yi2"},
{"guang1"},
{"hai3"},
{"ge1"},
{"dong4"},
{"----"},
{"jiao1"},
{"----"},
{"xiong1"},
{"----"},
{"an4"},
{"----"},
{"pian2"},
{"neng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi1"},
{"----"},
{"----"},
{"----"},
{"cui4"},
{"----"},
{"----"},
{"mai4"},
{"ji3"},
{"----"},
{"----"},
{"kuai4"},
{"----"},
{"zang4"},
{"qi2"},
{"nao3"},
{"----"},
{"nong2"},
{"luan2"},
{"----"},
{"bo2"},
{"----"},
{"----"},
{"----"},
{"jiao3"},
{"----"},
{"----"},
{"----"},
{"cuo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pao1"},
{"----"},
{"----"},
{"pu2"},
{"----"},
{"tuo1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tian3"},
{"----"},
{"jing1"},
{"----"},
{"la4"},
{"ye4"},
{"yan1"},
{"----"},
{"----"},
{"----"},
{"fu3"},
{"fu3"},
{"----"},
{"fei2"},
{"qiang1"},
{"wan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cou4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xing1"},
{"----"},
{"shu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"e4"},
{"sai1"},
{"----"},
{"yao1"},
{"jian4"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu4"},
{"xian4"},
{"ni4"},
{"----"},
{"wa4"},
{"teng2"},
{"tui3"},
{"bang3"},
{"----"},
{"lu:3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ge2"},
{"----"},
{"bo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gao1"},
{"----"},
{"bin4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"biao1"},
{"----"},
{"----"},
{"tang2"},
{"mo2"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"----"},
{"lin4"},
{"----"},
{"peng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ying1"},
{"shan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tun2"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"sao4"},
{"----"},
{"gu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chen2"},
{"----"},
{"----"},
{"----"},
{"zang1"},
{"----"},
{"----"},
{"zi4"},
{"----"},
{"nie4"},
{"xiu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"zhi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhen1"},
{"jiu4"},
{"----"},
{"yu2"},
{"----"},
{"yao3"},
{"yu2"},
{"chong1"},
{"----"},
{"xi4"},
{"jiu4"},
{"yu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"she2"},
{"she3"},
{"----"},
{"----"},
{"shi4"},
{"----"},
{"shu1"},
{"----"},
{"tian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chuan3"},
{"shun4"},
{"----"},
{"wu3"},
{"zhou1"},
{"----"},
{"----"},
{"shan1"},
{"yi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ban3"},
{"----"},
{"hang2"},
{"fang3"},
{"ban1"},
{"----"},
{"----"},
{"----"},
{"jian4"},
{"cang1"},
{"----"},
{"zhu2"},
{"ze2"},
{"duo4"},
{"bo2"},
{"xian2"},
{"ge3"},
{"chuan2"},
{"----"},
{"lu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shao1"},
{"----"},
{"----"},
{"ting3"},
{"----"},
{"----"},
{"----"},
{"meng3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sou1"},
{"----"},
{"cao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"meng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gen4"},
{"liang2"},
{"jian1"},
{"----"},
{"se4"},
{"yan4"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cao3"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"ai4"},
{"nai3"},
{"----"},
{"----"},
{"jie2"},
{"----"},
{"wan2"},
{"----"},
{"----"},
{"----"},
{"mi3"},
{"----"},
{"qian1"},
{"yu4"},
{"----"},
{"shao2"},
{"----"},
{"----"},
{"----"},
{"qi3"},
{"mang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiang1"},
{"pi2"},
{"fu2"},
{"----"},
{"----"},
{"wu2"},
{"zhi1"},
{"----"},
{"shan1"},
{"----"},
{"qian4"},
{"----"},
{"----"},
{"kou1"},
{"jie4"},
{"lu2"},
{"----"},
{"ji2"},
{"qin2"},
{"----"},
{"yuan2"},
{"fen1"},
{"ba1"},
{"rui4"},
{"xin1"},
{"ji4"},
{"hua1"},
{"----"},
{"fang1"},
{"wu4"},
{"----"},
{"----"},
{"zhi3"},
{"yun2"},
{"qin2"},
{"----"},
{"----"},
{"----"},
{"ya2"},
{"fei4"},
{"----"},
{"----"},
{"cong1"},
{"----"},
{"----"},
{"bian4"},
{"----"},
{"----"},
{"wei3"},
{"li4"},
{"----"},
{"----"},
{"xian4"},
{"chang2"},
{"cang1"},
{"zhu4"},
{"su1"},
{"----"},
{"yuan4"},
{"ran3"},
{"ling2"},
{"tai2"},
{"tiao2"},
{"----"},
{"miao2"},
{"----"},
{"----"},
{"----"},
{"ke1"},
{"mu4"},
{"----"},
{"bao1"},
{"gou3"},
{"min2"},
{"yi3"},
{"----"},
{"ju4"},
{"pie3"},
{"ruo4"},
{"ku3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shan1"},
{"----"},
{"----"},
{"----"},
{"ben3"},
{"----"},
{"ying1"},
{"----"},
{"----"},
{"ju1"},
{"----"},
{"----"},
{"gan1"},
{"----"},
{"ping2"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhuo2"},
{"mao4"},
{"fan4"},
{"qie2"},
{"mao2"},
{"----"},
{"ba2"},
{"ci2"},
{"mo4"},
{"----"},
{"----"},
{"chi2"},
{"----"},
{"jing1"},
{"long2"},
{"----"},
{"niao3"},
{"----"},
{"----"},
{"ying2"},
{"qiong2"},
{"----"},
{"ming2"},
{"----"},
{"----"},
{"----"},
{"gen4"},
{"qian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian3"},
{"ci2"},
{"----"},
{"----"},
{"mang2"},
{"cha2"},
{"jiao1"},
{"----"},
{"fu2"},
{"----"},
{"zhu1"},
{"----"},
{"jiang1"},
{"hui2"},
{"yin1"},
{"cha2"},
{"----"},
{"rong2"},
{"ru2"},
{"----"},
{"----"},
{"tong2"},
{"----"},
{"----"},
{"----"},
{"xun2"},
{"----"},
{"----"},
{"quan2"},
{"----"},
{"----"},
{"jing1"},
{"xing4"},
{"----"},
{"cao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ren3"},
{"jian4"},
{"yi2"},
{"huang1"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jia2"},
{"rao2"},
{"bi4"},
{"----"},
{"qiao2"},
{"hui4"},
{"qi2"},
{"dang4"},
{"----"},
{"rong2"},
{"hun1"},
{"xing2"},
{"luo4"},
{"ying2"},
{"----"},
{"jin4"},
{"sun1"},
{"yin4"},
{"----"},
{"hong2"},
{"----"},
{"yao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"he4"},
{"bi2"},
{"----"},
{"----"},
{"di2"},
{"tu2"},
{"sui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"pu2"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"suo1"},
{"----"},
{"----"},
{"----"},
{"ju3"},
{"mei2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shen1"},
{"----"},
{"----"},
{"ting2"},
{"----"},
{"----"},
{"wan3"},
{"----"},
{"you3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lang4"},
{"piao3"},
{"e2"},
{"mo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kan3"},
{"lai2"},
{"lian2"},
{"shi2"},
{"wo1"},
{"----"},
{"----"},
{"huo4"},
{"you2"},
{"ying2"},
{"ying1"},
{"----"},
{"chun2"},
{"mang3"},
{"----"},
{"----"},
{"wan3"},
{"jing1"},
{"----"},
{"----"},
{"----"},
{"jian1"},
{"----"},
{"gu1"},
{"----"},
{"----"},
{"ju2"},
{"----"},
{"jun4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"chang1"},
{"----"},
{"song1"},
{"----"},
{"----"},
{"----"},
{"cai4"},
{"ba2"},
{"----"},
{"tu4"},
{"bo1"},
{"han4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pu2"},
{"dang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gu1"},
{"ling2"},
{"fei1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi2"},
{"----"},
{"cui4"},
{"tao2"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi1"},
{"meng2"},
{"ping2"},
{"wei1"},
{"dan4"},
{"----"},
{"huan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"luo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ying2"},
{"ying2"},
{"ying2"},
{"xiao1"},
{"sa4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xuan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"----"},
{"e4"},
{"la4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"feng1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiang1"},
{"shen4"},
{"ge2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pu2"},
{"----"},
{"dong3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pa1"},
{"----"},
{"hu2"},
{"zang4"},
{"jia1"},
{"----"},
{"----"},
{"----"},
{"cong1"},
{"----"},
{"wei1"},
{"----"},
{"kui2"},
{"ting2"},
{"----"},
{"xi3"},
{"----"},
{"qi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"di4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chan3"},
{"----"},
{"kui4"},
{"----"},
{"jiang3"},
{"lou2"},
{"----"},
{"pai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lang4"},
{"----"},
{"meng1"},
{"----"},
{"----"},
{"suan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kuai3"},
{"----"},
{"----"},
{"pu2"},
{"----"},
{"shuo4"},
{"----"},
{"----"},
{"----"},
{"zheng1"},
{"jian1"},
{"ji2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rong2"},
{"weng3"},
{"----"},
{"----"},
{"shi1"},
{"----"},
{"----"},
{"ru4"},
{"----"},
{"----"},
{"bei4"},
{"----"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lan2"},
{"----"},
{"ji4"},
{"li2"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"mo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"peng2"},
{"----"},
{"----"},
{"----"},
{"xi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mie4"},
{"----"},
{"man4"},
{"----"},
{"----"},
{"----"},
{"zhe4"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cu4"},
{"----"},
{"cai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nian1"},
{"shu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiang2"},
{"----"},
{"----"},
{"lin4"},
{"----"},
{"ai3"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xun4"},
{"jiao1"},
{"rui3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qu2"},
{"----"},
{"----"},
{"hui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rui2"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi2"},
{"----"},
{"yun4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji2"},
{"hong4"},
{"----"},
{"----"},
{"lei3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bo2"},
{"hao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xue1"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"hong1"},
{"----"},
{"xin1"},
{"----"},
{"----"},
{"----"},
{"sou3"},
{"shu3"},
{"xun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ru2"},
{"----"},
{"tai2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cang2"},
{"miao3"},
{"----"},
{"----"},
{"xian3"},
{"----"},
{"ou3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"teng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zao3"},
{"----"},
{"----"},
{"----"},
{"huo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"heng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qu2"},
{"----"},
{"fan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhan4"},
{"----"},
{"----"},
{"----"},
{"mi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hu1"},
{"hu3"},
{"lu3"},
{"nu:e4"},
{"lu:4"},
{"----"},
{"----"},
{"qian2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xu1"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"----"},
{"guo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chong2"},
{"qiu2"},
{"----"},
{"ji3"},
{"----"},
{"----"},
{"shi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jiang4"},
{"hui1"},
{"meng2"},
{"ge4"},
{"sui1"},
{"xia1"},
{"chai4"},
{"shi2"},
{"yi3"},
{"ma3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wen2"},
{"rui4"},
{"bang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yin3"},
{"----"},
{"can2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ya2"},
{"hao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gong1"},
{"zao3"},
{"----"},
{"----"},
{"jie4"},
{"fu2"},
{"chi1"},
{"dou3"},
{"----"},
{"xian3"},
{"----"},
{"----"},
{"qiu1"},
{"you2"},
{"zha4"},
{"----"},
{"----"},
{"you4"},
{"e2"},
{"han1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhu4"},
{"----"},
{"----"},
{"----"},
{"gu1"},
{"----"},
{"qu1"},
{"she2"},
{"----"},
{"ling2"},
{"gu3"},
{"dan4"},
{"----"},
{"----"},
{"li4"},
{"cheng1"},
{"qu1"},
{"mou2"},
{"----"},
{"----"},
{"hui2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wa1"},
{"----"},
{"zhu1"},
{"----"},
{"----"},
{"----"},
{"jiao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ha2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiong2"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"man2"},
{"----"},
{"zhi2"},
{"jia2"},
{"----"},
{"si1"},
{"qi2"},
{"----"},
{"----"},
{"----"},
{"shao1"},
{"yong3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"e2"},
{"----"},
{"shu3"},
{"----"},
{"feng1"},
{"shen4"},
{"----"},
{"----"},
{"----"},
{"zhe1"},
{"wu2"},
{"fu2"},
{"li2"},
{"----"},
{"----"},
{"chu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan2"},
{"ting2"},
{"----"},
{"tui4"},
{"----"},
{"----"},
{"zhi1"},
{"----"},
{"fei3"},
{"----"},
{"mi4"},
{"----"},
{"qi2"},
{"----"},
{"----"},
{"la4"},
{"meng3"},
{"qiang1"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"tiao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"pi2"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"quan2"},
{"----"},
{"----"},
{"----"},
{"qing1"},
{"----"},
{"----"},
{"guo3"},
{"wan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ying2"},
{"guo1"},
{"chan2"},
{"----"},
{"----"},
{"ke1"},
{"----"},
{"xie1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"bian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"you2"},
{"qiu2"},
{"mao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hu2"},
{"----"},
{"die2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nan3"},
{"lou2"},
{"----"},
{"rong2"},
{"----"},
{"----"},
{"----"},
{"lang2"},
{"pang2"},
{"----"},
{"xi1"},
{"----"},
{"----"},
{"yuan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"rong2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qin2"},
{"----"},
{"----"},
{"----"},
{"tang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ming2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhe1"},
{"cao2"},
{"chi1"},
{"----"},
{"ao2"},
{"----"},
{"----"},
{"----"},
{"tang2"},
{"----"},
{"piao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"luo2"},
{"----"},
{"----"},
{"zhong1"},
{"----"},
{"----"},
{"shuai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ma5"},
{"----"},
{"----"},
{"----"},
{"mao2"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhang1"},
{"mang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"peng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"luo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lian2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"meng3"},
{"----"},
{"ru2"},
{"huo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li3"},
{"chun3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"juan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"du4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xue4"},
{"----"},
{"----"},
{"----"},
{"nu:4"},
{"xin4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xing2"},
{"yan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xian2"},
{"----"},
{"----"},
{"jie1"},
{"----"},
{"ya2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"heng2"},
{"qu2"},
{"yi1"},
{"----"},
{"bu3"},
{"----"},
{"----"},
{"biao3"},
{"cha3"},
{"----"},
{"shan1"},
{"chen4"},
{"----"},
{"gun3"},
{"----"},
{"shuai1"},
{"----"},
{"na4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ren4"},
{"qin1"},
{"jin1"},
{"----"},
{"yuan2"},
{"mei4"},
{"----"},
{"ao3"},
{"niao3"},
{"----"},
{"----"},
{"jia1"},
{"----"},
{"----"},
{"dai4"},
{"----"},
{"pao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tan3"},
{"----"},
{"----"},
{"----"},
{"xiu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wa4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bei4"},
{"----"},
{"xi2"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jia2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cai2"},
{"lie4"},
{"----"},
{"----"},
{"zhuang1"},
{"dang1"},
{"----"},
{"----"},
{"ken4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cheng2"},
{"----"},
{"----"},
{"----"},
{"pou2"},
{"----"},
{"yi4"},
{"yu4"},
{"----"},
{"----"},
{"qiu2"},
{"qun2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sha1"},
{"----"},
{"----"},
{"lian2"},
{"----"},
{"ku4"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"biao3"},
{"----"},
{"chang2"},
{"pei2"},
{"----"},
{"----"},
{"----"},
{"luo3"},
{"guo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ju1"},
{"----"},
{"----"},
{"----"},
{"gua4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"he4"},
{"----"},
{"bao1"},
{"bao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bei4"},
{"chu3"},
{"lu:3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"da1"},
{"----"},
{"----"},
{"----"},
{"ru4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tun4"},
{"chi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qian1"},
{"----"},
{"----"},
{"----"},
{"lan2"},
{"----"},
{"zhe3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiang3"},
{"----"},
{"----"},
{"xiang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"jin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ru2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pan4"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"yao4"},
{"----"},
{"tan2"},
{"----"},
{"----"},
{"fu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian4"},
{"guan1"},
{"----"},
{"gui1"},
{"mi4"},
{"shi4"},
{"zhan1"},
{"lan3"},
{"jue2"},
{"ji4"},
{"xi2"},
{"di2"},
{"----"},
{"yu2"},
{"gou4"},
{"jin4"},
{"qu4"},
{"jiao3"},
{"----"},
{"----"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"gu1"},
{"----"},
{"zui3"},
{"----"},
{"shang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jie3"},
{"----"},
{"gong1"},
{"chu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"su4"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"----"},
{"----"},
{"hu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"teng2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jing3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji4"},
{"ding4"},
{"fu4"},
{"ren4"},
{"ji1"},
{"jie2"},
{"----"},
{"tao3"},
{"rang4"},
{"shan4"},
{"qi4"},
{"----"},
{"xun4"},
{"yi4"},
{"xun4"},
{"ji4"},
{"----"},
{"jiang3"},
{"hui4"},
{"ou1"},
{"ju4"},
{"ya4"},
{"ne4"},
{"xu3"},
{"e2"},
{"lun4"},
{"----"},
{"song4"},
{"feng4"},
{"she4"},
{"fang3"},
{"jue2"},
{"zheng4"},
{"gu3"},
{"he1"},
{"ping2"},
{"zu3"},
{"shi2"},
{"----"},
{"zha4"},
{"su4"},
{"zhen3"},
{"di3"},
{"zou1"},
{"ci2"},
{"----"},
{"zhao4"},
{"----"},
{"yi4"},
{"yi2"},
{"kuang1"},
{"lei3"},
{"shi4"},
{"gua4"},
{"shi1"},
{"jie2"},
{"hui1"},
{"cheng2"},
{"zhu1"},
{"shen1"},
{"hua4"},
{"dan4"},
{"gou4"},
{"quan2"},
{"gui3"},
{"xun2"},
{"yi4"},
{"----"},
{"gai1"},
{"xiang2"},
{"cha4"},
{"hun4"},
{"xu3"},
{"----"},
{"jie4"},
{"wu1"},
{"yu3"},
{"qiao4"},
{"wu4"},
{"gao4"},
{"you4"},
{"hui4"},
{"kuang2"},
{"shuo1"},
{"song4"},
{"----"},
{"qing3"},
{"zhu1"},
{"zou1"},
{"nuo4"},
{"du2"},
{"zhuo2"},
{"fei3"},
{"ke4"},
{"wei3"},
{"yu2"},
{"shui2"},
{"shen3"},
{"tiao2"},
{"chan3"},
{"liang4"},
{"zhun1"},
{"sui4"},
{"tan2"},
{"----"},
{"yi4"},
{"mou2"},
{"chen2"},
{"die2"},
{"huang3"},
{"jian4"},
{"xie2"},
{"----"},
{"ye4"},
{"wei4"},
{"e4"},
{"yu4"},
{"xuan1"},
{"chan2"},
{"zi1"},
{"an1"},
{"yan4"},
{"di4"},
{"mi2"},
{"pian3"},
{"----"},
{"mo2"},
{"dang3"},
{"su4"},
{"xie4"},
{"yao2"},
{"bang4"},
{"shi4"},
{"qian1"},
{"mi4"},
{"jin3"},
{"man4"},
{"zhe2"},
{"jian3"},
{"miu4"},
{"tan2"},
{"zen4"},
{"qiao4"},
{"lan2"},
{"pu3"},
{"jue2"},
{"yan4"},
{"qian3"},
{"zhan1"},
{"chen4"},
{"gu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dou4"},
{"jiang1"},
{"----"},
{"chi3"},
{"----"},
{"----"},
{"wan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tun2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiang4"},
{"huan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hao2"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"bao4"},
{"chai2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"diao1"},
{"----"},
{"----"},
{"xiu1"},
{"----"},
{"----"},
{"----"},
{"he2"},
{"mo4"},
{"----"},
{"mao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi2"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bei4"},
{"zhen1"},
{"fu4"},
{"----"},
{"gong4"},
{"cai2"},
{"ze2"},
{"xian2"},
{"bai4"},
{"zhang4"},
{"huo4"},
{"zhi4"},
{"fan4"},
{"tan1"},
{"pin2"},
{"bian3"},
{"gou4"},
{"zhu4"},
{"guan4"},
{"er4"},
{"jian4"},
{"bi4"},
{"shi4"},
{"tie1"},
{"gui4"},
{"kuang4"},
{"dai4"},
{"mao4"},
{"fei4"},
{"he4"},
{"yi2"},
{"zei2"},
{"zhi4"},
{"gu3"},
{"hui4"},
{"zi1"},
{"lin4"},
{"lu4"},
{"zang1"},
{"zi1"},
{"gai1"},
{"jin4"},
{"qiu2"},
{"zhen4"},
{"lai4"},
{"she1"},
{"fu4"},
{"du3"},
{"ji1"},
{"shu2"},
{"shang3"},
{"ci4"},
{"----"},
{"----"},
{"geng1"},
{"pei2"},
{"----"},
{"lai4"},
{"----"},
{"zhui4"},
{"fu4"},
{"zhuan4"},
{"sai4"},
{"ze2"},
{"yan4"},
{"zan4"},
{"----"},
{"zeng4"},
{"shan4"},
{"ying2"},
{"gan4"},
{"chi4"},
{"----"},
{"she4"},
{"nan3"},
{"----"},
{"----"},
{"----"},
{"he4"},
{"----"},
{"zhe3"},
{"----"},
{"----"},
{"zou3"},
{"----"},
{"----"},
{"----"},
{"fu4"},
{"zhao4"},
{"gan3"},
{"qi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chen4"},
{"----"},
{"----"},
{"qie4"},
{"chao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yue4"},
{"qu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tang4"},
{"----"},
{"----"},
{"----"},
{"qu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zan3"},
{"----"},
{"zu2"},
{"pa1"},
{"bao4"},
{"----"},
{"----"},
{"dun3"},
{"----"},
{"fu1"},
{"----"},
{"jian3"},
{"----"},
{"zhi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yue4"},
{"qiang1"},
{"----"},
{"tai2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ba2"},
{"die1"},
{"----"},
{"tuo2"},
{"jia1"},
{"----"},
{"pao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi2"},
{"fu1"},
{"----"},
{"----"},
{"shan1"},
{"bo3"},
{"----"},
{"ju4"},
{"li4"},
{"gen1"},
{"----"},
{"----"},
{"----"},
{"xian3"},
{"jiao1"},
{"----"},
{"----"},
{"----"},
{"kua4"},
{"----"},
{"gui4"},
{"qiong2"},
{"kui3"},
{"----"},
{"----"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"tiao4"},
{"----"},
{"jian4"},
{"----"},
{"qiao1"},
{"bi4"},
{"xian1"},
{"duo4"},
{"ji1"},
{"----"},
{"ji4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xue2"},
{"----"},
{"----"},
{"----"},
{"liang4"},
{"yong3"},
{"----"},
{"chou2"},
{"----"},
{"----"},
{"ta4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huai2"},
{"ju4"},
{"chi2"},
{"----"},
{"----"},
{"ti1"},
{"bo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cai3"},
{"zong1"},
{"----"},
{"zhi4"},
{"----"},
{"dian3"},
{"zhi2"},
{"----"},
{"duo2"},
{"----"},
{"----"},
{"----"},
{"zhong3"},
{"----"},
{"----"},
{"----"},
{"chuai4"},
{"----"},
{"----"},
{"----"},
{"ju3"},
{"----"},
{"----"},
{"die2"},
{"pian2"},
{"rou2"},
{"----"},
{"ti2"},
{"----"},
{"----"},
{"jian3"},
{"dao3"},
{"cuo1"},
{"xi1"},
{"ta4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"nie4"},
{"pan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"beng4"},
{"----"},
{"----"},
{"bie2"},
{"----"},
{"----"},
{"deng4"},
{"ceng4"},
{"----"},
{"fan2"},
{"chu2"},
{"----"},
{"dun1"},
{"----"},
{"cu4"},
{"----"},
{"jue2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pu3"},
{"----"},
{"----"},
{"cuan1"},
{"----"},
{"zao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lie4"},
{"----"},
{"----"},
{"----"},
{"chan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shen1"},
{"gong1"},
{"----"},
{"----"},
{"qu1"},
{"----"},
{"----"},
{"duo3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"che1"},
{"ya4"},
{"gui3"},
{"xuan1"},
{"----"},
{"ren4"},
{"zhuan4"},
{"e4"},
{"lun2"},
{"ruan3"},
{"hong1"},
{"----"},
{"ke1"},
{"lu2"},
{"zhu2"},
{"zhi3"},
{"yi4"},
{"----"},
{"zhen3"},
{"li4"},
{"yao2"},
{"qing1"},
{"shi4"},
{"zai4"},
{"zhi4"},
{"jiao4"},
{"----"},
{"quan2"},
{"lu4"},
{"jiao4"},
{"zhe2"},
{"fu3"},
{"liang4"},
{"nian3"},
{"bei4"},
{"hui1"},
{"gun3"},
{"wang3"},
{"----"},
{"chuo4"},
{"zi1"},
{"cou4"},
{"fu2"},
{"ji2"},
{"----"},
{"shu1"},
{"pei4"},
{"yuan2"},
{"xia2"},
{"zhan3"},
{"lu4"},
{"che4"},
{"lin2"},
{"xin1"},
{"gu1"},
{"----"},
{"ci2"},
{"pi4"},
{"----"},
{"----"},
{"----"},
{"la4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bian4"},
{"bian4"},
{"----"},
{"bian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chen2"},
{"ru3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bian1"},
{"----"},
{"----"},
{"----"},
{"liao2"},
{"da2"},
{"----"},
{"----"},
{"qian1"},
{"yu1"},
{"----"},
{"qi4"},
{"xun4"},
{"----"},
{"guo4"},
{"mai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ying2"},
{"----"},
{"yun4"},
{"jin4"},
{"----"},
{"ya4"},
{"fan3"},
{"----"},
{"----"},
{"----"},
{"hai2"},
{"zhe4"},
{"----"},
{"jin4"},
{"yuan3"},
{"wei2"},
{"lian2"},
{"chi2"},
{"----"},
{"----"},
{"tiao2"},
{"----"},
{"yi3"},
{"jiong3"},
{"jia1"},
{"----"},
{"dai4"},
{"er3"},
{"di2"},
{"po4"},
{"----"},
{"die2"},
{"ze2"},
{"----"},
{"shu4"},
{"----"},
{"----"},
{"jing4"},
{"----"},
{"----"},
{"----"},
{"mi2"},
{"beng4"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"zhui1"},
{"----"},
{"----"},
{"tui4"},
{"song4"},
{"shi4"},
{"tao2"},
{"pang2"},
{"hou4"},
{"ni4"},
{"----"},
{"----"},
{"xuan3"},
{"xun4"},
{"bu1"},
{"----"},
{"xiao1"},
{"----"},
{"tou4"},
{"zhu2"},
{"qiu2"},
{"di4"},
{"----"},
{"tu2"},
{"----"},
{"ti4"},
{"dou4"},
{"----"},
{"----"},
{"tong1"},
{"guang4"},
{"----"},
{"shi4"},
{"cheng3"},
{"su4"},
{"zao4"},
{"qun1"},
{"feng2"},
{"----"},
{"----"},
{"----"},
{"li3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huan4"},
{"dai4"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kui2"},
{"wei1"},
{"----"},
{"yi4"},
{"----"},
{"----"},
{"luo2"},
{"bi1"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"dun4"},
{"sui4"},
{"----"},
{"chuan2"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bian4"},
{"----"},
{"e4"},
{"xia2"},
{"huang2"},
{"qiu2"},
{"dao4"},
{"----"},
{"----"},
{"----"},
{"yi2"},
{"gou4"},
{"----"},
{"----"},
{"liu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ta4"},
{"qian3"},
{"----"},
{"yao2"},
{"----"},
{"----"},
{"ao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zao1"},
{"zhe1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lin2"},
{"zun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ju4"},
{"----"},
{"bi4"},
{"yao1"},
{"----"},
{"xie4"},
{"sui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"miao3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yi4"},
{"----"},
{"deng4"},
{"----"},
{"yong1"},
{"----"},
{"han2"},
{"----"},
{"mang2"},
{"----"},
{"qiong2"},
{"----"},
{"kuang4"},
{"----"},
{"----"},
{"----"},
{"fang1"},
{"xing2"},
{"nei4"},
{"----"},
{"----"},
{"bang1"},
{"----"},
{"----"},
{"----"},
{"xie2"},
{"----"},
{"wu1"},
{"----"},
{"you2"},
{"han2"},
{"tai2"},
{"qiu1"},
{"----"},
{"----"},
{"bing3"},
{"shao4"},
{"bei4"},
{"----"},
{"di3"},
{"zou1"},
{"ye4"},
{"lin2"},
{"----"},
{"----"},
{"zhu1"},
{"----"},
{"----"},
{"yu4"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"xun2"},
{"----"},
{"----"},
{"jiao1"},
{"----"},
{"----"},
{"----"},
{"lang2"},
{"jia2"},
{"kuai4"},
{"zheng4"},
{"----"},
{"yun4"},
{"----"},
{"----"},
{"----"},
{"chi1"},
{"----"},
{"----"},
{"----"},
{"fu2"},
{"gao4"},
{"hao3"},
{"----"},
{"----"},
{"----"},
{"jun4"},
{"ying3"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"yun2"},
{"bu4"},
{"----"},
{"----"},
{"pi2"},
{"----"},
{"guo1"},
{"----"},
{"tan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chen1"},
{"----"},
{"----"},
{"----"},
{"dan1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"dou1"},
{"yan3"},
{"----"},
{"----"},
{"----"},
{"e4"},
{"----"},
{"juan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yin2"},
{"----"},
{"----"},
{"----"},
{"yan1"},
{"zhang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shan4"},
{"----"},
{"po2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ling2"},
{"----"},
{"----"},
{"feng1"},
{"----"},
{"----"},
{"you3"},
{"ding3"},
{"qiu2"},
{"zhuo2"},
{"pei4"},
{"zhou4"},
{"yi2"},
{"gan1"},
{"----"},
{"jiu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xu4"},
{"----"},
{"----"},
{"fen1"},
{"----"},
{"----"},
{"yun4"},
{"tai4"},
{"----"},
{"----"},
{"tuo2"},
{"zuo4"},
{"han1"},
{"gu1"},
{"su1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lao4"},
{"----"},
{"chou2"},
{"----"},
{"tong2"},
{"zhi3"},
{"xian1"},
{"jiang4"},
{"cheng2"},
{"----"},
{"tu2"},
{"jiao4"},
{"mei2"},
{"ku4"},
{"suan1"},
{"lei4"},
{"----"},
{"----"},
{"----"},
{"yan4"},
{"----"},
{"niang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pei1"},
{"----"},
{"chun2"},
{"----"},
{"zui4"},
{"----"},
{"cu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hu2"},
{"xu3"},
{"xing3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mi2"},
{"quan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"hai3"},
{"tang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"lao2"},
{"----"},
{"----"},
{"bu2"},
{"jiao4"},
{"xi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"li3"},
{"ju4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cai3"},
{"----"},
{"you4"},
{"shi4"},
{"----"},
{"li3"},
{"zhong4"},
{"ye3"},
{"liang4"},
{"----"},
{"jin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"luan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"liu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ga2"},
{"yi3"},
{"zhen1"},
{"ding1"},
{"zhao1"},
{"po1"},
{"liao3"},
{"tu3"},
{"qian1"},
{"chuan4"},
{"shan1"},
{"----"},
{"fan2"},
{"diao4"},
{"men2"},
{"nu:3"},
{"----"},
{"chai1"},
{"----"},
{"gai4"},
{"bu4"},
{"tai4"},
{"ju4"},
{"dun4"},
{"chao1"},
{"zhong1"},
{"na4"},
{"bei4"},
{"gang1"},
{"ban3"},
{"qian2"},
{"yao4"},
{"qin1"},
{"jun1"},
{"wu1"},
{"gou1"},
{"kang4"},
{"fang1"},
{"huo3"},
{"tou3"},
{"niu3"},
{"ba3"},
{"yu4"},
{"qian2"},
{"zheng1"},
{"qian2"},
{"gu3"},
{"bo1"},
{"ke1"},
{"----"},
{"----"},
{"bo2"},
{"yue4"},
{"zuan1"},
{"mu4"},
{"----"},
{"jia3"},
{"tian2"},
{"you2"},
{"tie3"},
{"bo2"},
{"ling2"},
{"shuo4"},
{"qian1"},
{"mao3"},
{"----"},
{"shi4"},
{"xuan4"},
{"ta1"},
{"bi4"},
{"ni2"},
{"pi2"},
{"duo2"},
{"----"},
{"kao4"},
{"lao3"},
{"er3"},
{"----"},
{"----"},
{"you3"},
{"cheng2"},
{"jia2"},
{"----"},
{"nao2"},
{"----"},
{"cheng1"},
{"tong2"},
{"lu:3"},
{"----"},
{"yin1"},
{"kai3"},
{"zha2"},
{"zhu1"},
{"xian3"},
{"ting3"},
{"diu1"},
{"----"},
{"hua2"},
{"quan2"},
{"sha1"},
{"ha1"},
{"yao2"},
{"ge4"},
{"ming2"},
{"zheng1"},
{"se4"},
{"jiao3"},
{"yi1"},
{"chan3"},
{"chong4"},
{"----"},
{"an3"},
{"yin2"},
{"ru2"},
{"zhu4"},
{"lao2"},
{"pu4"},
{"----"},
{"lai2"},
{"te4"},
{"lian4"},
{"keng1"},
{"xiao1"},
{"suo3"},
{"li3"},
{"----"},
{"chu2"},
{"guo1"},
{"gao4"},
{"e2"},
{"xiu4"},
{"cuo4"},
{"lu:e4"},
{"feng1"},
{"xin1"},
{"----"},
{"----"},
{"jian3"},
{"rui4"},
{"ti1"},
{"lang2"},
{"----"},
{"ju1"},
{"----"},
{"qiang1"},
{"zhe3"},
{"----"},
{"cuo4"},
{"mao2"},
{"ben1"},
{"----"},
{"----"},
{"ke4"},
{"kun1"},
{"----"},
{"xi2"},
{"gu4"},
{"luo2"},
{"chui2"},
{"zhui1"},
{"jin3"},
{"----"},
{"----"},
{"juan4"},
{"----"},
{"----"},
{"tan2"},
{"ding4"},
{"jian4"},
{"ju4"},
{"meng3"},
{"zi1"},
{"qie4"},
{"----"},
{"kai3"},
{"qiang1"},
{"si1"},
{"e4"},
{"----"},
{"qiao1"},
{"zhong1"},
{"duan4"},
{"sou1"},
{"----"},
{"huan2"},
{"----"},
{"du4"},
{"mei3"},
{"lou4"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"zhen4"},
{"----"},
{"ge2"},
{"nie4"},
{"----"},
{"juan1"},
{"nie4"},
{"----"},
{"liu2"},
{"hao4"},
{"bang4"},
{"yi4"},
{"jia1"},
{"bin1"},
{"----"},
{"biao1"},
{"tang1"},
{"man4"},
{"----"},
{"----"},
{"yong1"},
{"jing4"},
{"di1"},
{"zu2"},
{"xuan4"},
{"----"},
{"tan2"},
{"----"},
{"----"},
{"pu2"},
{"----"},
{"dun1"},
{"lan2"},
{"pu3"},
{"----"},
{"qiang3"},
{"deng4"},
{"huo4"},
{"lei2"},
{"----"},
{"zhuo2"},
{"lian2"},
{"yi4"},
{"----"},
{"biao1"},
{"----"},
{"----"},
{"xiang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"men2"},
{"shuan1"},
{"shan3"},
{"yan2"},
{"----"},
{"bi4"},
{"wen4"},
{"chuang3"},
{"run4"},
{"wei2"},
{"xian2"},
{"hong2"},
{"jian4"},
{"min3"},
{"----"},
{"men4"},
{"zha2"},
{"nao4"},
{"gui1"},
{"wen2"},
{"ta4"},
{"min3"},
{"lu:2"},
{"----"},
{"fa2"},
{"ge2"},
{"he2"},
{"kun3"},
{"jiu1"},
{"yue4"},
{"----"},
{"----"},
{"yu4"},
{"yan1"},
{"chang1"},
{"xi4"},
{"wen2"},
{"hun1"},
{"yan2"},
{"yan1"},
{"chan3"},
{"lan2"},
{"qu4"},
{"----"},
{"kuo4"},
{"que4"},
{"ge2"},
{"tian2"},
{"----"},
{"que1"},
{"kan4"},
{"----"},
{"fu4"},
{"----"},
{"----"},
{"dui4"},
{"----"},
{"qian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ban3"},
{"----"},
{"----"},
{"----"},
{"ruan3"},
{"----"},
{"----"},
{"jing3"},
{"fang2"},
{"yang2"},
{"yin1"},
{"zhen4"},
{"jie1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zu3"},
{"zuo4"},
{"yan2"},
{"----"},
{"a1"},
{"tuo2"},
{"----"},
{"po1"},
{"----"},
{"fu4"},
{"ji4"},
{"liu4"},
{"long3"},
{"chen2"},
{"xing2"},
{"----"},
{"lou4"},
{"mo4"},
{"jiang4"},
{"----"},
{"----"},
{"xian4"},
{"----"},
{"----"},
{"----"},
{"gai1"},
{"shan3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"----"},
{"----"},
{"----"},
{"zhi4"},
{"----"},
{"dou3"},
{"yuan4"},
{"----"},
{"chu2"},
{"----"},
{"----"},
{"nie4"},
{"yun3"},
{"xian3"},
{"pei2"},
{"----"},
{"zou1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chui2"},
{"----"},
{"pi2"},
{"ling2"},
{"tao2"},
{"xian4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"long2"},
{"----"},
{"wei1"},
{"----"},
{"----"},
{"sui2"},
{"----"},
{"huang2"},
{"----"},
{"sui2"},
{"yin3"},
{"----"},
{"----"},
{"----"},
{"ge2"},
{"----"},
{"----"},
{"wei3"},
{"ai4"},
{"xi4"},
{"----"},
{"----"},
{"zhang4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sui4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi2"},
{"----"},
{"----"},
{"hui1"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"zhui1"},
{"----"},
{"----"},
{"sun3"},
{"jun4"},
{"nan2"},
{"----"},
{"qiao1"},
{"yan4"},
{"----"},
{"----"},
{"xiong2"},
{"ya3"},
{"ji2"},
{"gu4"},
{"----"},
{"zhi4"},
{"----"},
{"----"},
{"ci2"},
{"yong1"},
{"ju1"},
{"chu2"},
{"----"},
{"----"},
{"luo4"},
{"----"},
{"----"},
{"diao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu3"},
{"yu2"},
{"xue3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wen2"},
{"----"},
{"----"},
{"----"},
{"li4"},
{"----"},
{"----"},
{"ling2"},
{"lei2"},
{"----"},
{"bao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wu4"},
{"----"},
{"xu1"},
{"ji4"},
{"----"},
{"----"},
{"xiao1"},
{"----"},
{"ting2"},
{"zhen4"},
{"pei4"},
{"mei2"},
{"----"},
{"----"},
{"----"},
{"huo4"},
{"sha4"},
{"fei1"},
{"----"},
{"----"},
{"----"},
{"ni2"},
{"----"},
{"----"},
{"lin2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shuang1"},
{"----"},
{"xia2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yin2"},
{"----"},
{"----"},
{"ai3"},
{"----"},
{"----"},
{"xian4"},
{"----"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ba4"},
{"pi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mai2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qing1"},
{"jing4"},
{"----"},
{"----"},
{"jing4"},
{"----"},
{"----"},
{"jing4"},
{"----"},
{"dian4"},
{"----"},
{"----"},
{"fei1"},
{"----"},
{"kao4"},
{"mi3"},
{"mian4"},
{"----"},
{"----"},
{"ye4"},
{"----"},
{"----"},
{"----"},
{"ge2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jin4"},
{"xue1"},
{"----"},
{"ba3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"da2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xie2"},
{"----"},
{"an1"},
{"----"},
{"----"},
{"----"},
{"da2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qiao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ju1"},
{"----"},
{"----"},
{"rou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bian1"},
{"----"},
{"jian1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"wei2"},
{"ren4"},
{"----"},
{"han2"},
{"wei3"},
{"yun4"},
{"tao1"},
{"jiu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yin1"},
{"----"},
{"yun4"},
{"shao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ye4"},
{"ding3"},
{"qing3"},
{"han1"},
{"xiang4"},
{"shun4"},
{"xu1"},
{"xu4"},
{"wan2"},
{"gu4"},
{"dun4"},
{"qi2"},
{"ban1"},
{"song4"},
{"hang2"},
{"yu4"},
{"lu2"},
{"ling3"},
{"po1"},
{"jing3"},
{"xie2"},
{"jia2"},
{"----"},
{"he2"},
{"----"},
{"----"},
{"ke2"},
{"yi2"},
{"pin2"},
{"----"},
{"tui2"},
{"han4"},
{"----"},
{"ying3"},
{"ke1"},
{"ti2"},
{"----"},
{"e4"},
{"zhuan1"},
{"yan2"},
{"e2"},
{"nie4"},
{"man1"},
{"dian1"},
{"sang3"},
{"hao4"},
{"----"},
{"zhan4"},
{"----"},
{"pin2"},
{"quan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"feng1"},
{"----"},
{"----"},
{"biao1"},
{"sa4"},
{"ju4"},
{"----"},
{"sou1"},
{"----"},
{"----"},
{"piao1"},
{"biao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fei1"},
{"shi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sun1"},
{"xiang3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yan4"},
{"----"},
{"----"},
{"can1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yong1"},
{"tao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji1"},
{"----"},
{"xing2"},
{"tun2"},
{"xi4"},
{"ren4"},
{"yu4"},
{"chi4"},
{"fan4"},
{"yin3"},
{"jian4"},
{"shi4"},
{"bao3"},
{"si4"},
{"----"},
{"yi2"},
{"er3"},
{"rao2"},
{"xiang3"},
{"----"},
{"----"},
{"jiao3"},
{"----"},
{"bing3"},
{"bo1"},
{"----"},
{"e4"},
{"yu2"},
{"nei3"},
{"----"},
{"----"},
{"hun2"},
{"xian4"},
{"guan3"},
{"----"},
{"kui4"},
{"----"},
{"sou1"},
{"chan2"},
{"----"},
{"mo2"},
{"----"},
{"liu4"},
{"xiu1"},
{"jin3"},
{"man2"},
{"----"},
{"zhuan4"},
{"----"},
{"shou3"},
{"kui2"},
{"guo2"},
{"xiang1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xin1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ma3"},
{"yu4"},
{"tuo2"},
{"xun2"},
{"chi2"},
{"qu1"},
{"----"},
{"bo2"},
{"lu:2"},
{"zang3"},
{"shi3"},
{"si4"},
{"fu4"},
{"ju1"},
{"zou1"},
{"zhu4"},
{"tuo2"},
{"nu2"},
{"jia4"},
{"yi4"},
{"tai2"},
{"xiao1"},
{"ma4"},
{"----"},
{"jiao1"},
{"hua2"},
{"luo4"},
{"hai4"},
{"pian2"},
{"----"},
{"li2"},
{"cheng3"},
{"yan4"},
{"----"},
{"----"},
{"jun4"},
{"qi2"},
{"qi2"},
{"ke4"},
{"zhui1"},
{"----"},
{"----"},
{"can1"},
{"pian4"},
{"zhi4"},
{"----"},
{"sao1"},
{"wu4"},
{"ao4"},
{"liu2"},
{"qian1"},
{"shan4"},
{"piao4"},
{"luo2"},
{"cong1"},
{"----"},
{"zhou4"},
{"ji4"},
{"----"},
{"xiang1"},
{"gu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tou2"},
{"xie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"di3"},
{"ku1"},
{"hai2"},
{"----"},
{"----"},
{"----"},
{"ge2"},
{"----"},
{"----"},
{"----"},
{"bi4"},
{"ke1"},
{"qia4"},
{"----"},
{"----"},
{"lou2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kuan1"},
{"bin4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"du2"},
{"----"},
{"sui3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gao1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"biao1"},
{"----"},
{"kun1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mao2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"tiao2"},
{"----"},
{"zi1"},
{"----"},
{"ran2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xiu1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zong1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"quan2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"bin4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huan2"},
{"----"},
{"----"},
{"----"},
{"lie4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chang4"},
{"----"},
{"----"},
{"ge2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu4"},
{"gui3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"kui2"},
{"hun2"},
{"ba2"},
{"po4"},
{"mei4"},
{"----"},
{"yan3"},
{"xiao1"},
{"liang3"},
{"----"},
{"----"},
{"----"},
{"wang3"},
{"----"},
{"wei4"},
{"----"},
{"chi1"},
{"----"},
{"----"},
{"mo2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yu2"},
{"----"},
{"----"},
{"you2"},
{"----"},
{"lu3"},
{"fang2"},
{"----"},
{"----"},
{"ba4"},
{"ping2"},
{"----"},
{"lu2"},
{"----"},
{"----"},
{"fu4"},
{"----"},
{"bao4"},
{"hou4"},
{"----"},
{"tai2"},
{"gui1"},
{"jie2"},
{"----"},
{"wei3"},
{"er2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"ji4"},
{"jiao1"},
{"xian3"},
{"----"},
{"xiang3"},
{"xun2"},
{"geng3"},
{"----"},
{"lian2"},
{"jian1"},
{"li3"},
{"shi2"},
{"tiao2"},
{"gun3"},
{"sha1"},
{"----"},
{"----"},
{"ji4"},
{"----"},
{"qing1"},
{"ling2"},
{"----"},
{"zou1"},
{"fei1"},
{"kun1"},
{"chang1"},
{"----"},
{"ni2"},
{"nian2"},
{"diao1"},
{"jing1"},
{"----"},
{"----"},
{"zi1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"sai1"},
{"e4"},
{"qiu1"},
{"fu4"},
{"huang2"},
{"----"},
{"----"},
{"bian1"},
{"----"},
{"ao2"},
{"qi2"},
{"ta3"},
{"guan1"},
{"yao2"},
{"----"},
{"----"},
{"le4"},
{"biao4"},
{"xue3"},
{"bie1"},
{"man2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"gui4"},
{"shan4"},
{"lin2"},
{"----"},
{"----"},
{"----"},
{"li3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"niao3"},
{"jiu1"},
{"ji1"},
{"yuan1"},
{"ming2"},
{"----"},
{"ou1"},
{"ya1"},
{"----"},
{"bao3"},
{"zhen4"},
{"gu1"},
{"dong1"},
{"lu2"},
{"ya1"},
{"----"},
{"yang1"},
{"----"},
{"chi1"},
{"qu2"},
{"yuan1"},
{"----"},
{"tuo2"},
{"si1"},
{"zhi4"},
{"er2"},
{"gua1"},
{"xiu1"},
{"----"},
{"----"},
{"ge1"},
{"luan2"},
{"hong2"},
{"----"},
{"bo2"},
{"li2"},
{"juan1"},
{"gu3"},
{"e2"},
{"yu4"},
{"----"},
{"ti2"},
{"wu3"},
{"que4"},
{"----"},
{"an1"},
{"----"},
{"bei1"},
{"peng2"},
{"----"},
{"chun2"},
{"----"},
{"----"},
{"----"},
{"hu2"},
{"----"},
{"e4"},
{"hu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yao4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jian1"},
{"he4"},
{"----"},
{"ying1"},
{"zhe4"},
{"----"},
{"liao2"},
{"jiao1"},
{"jiu4"},
{"yu4"},
{"lu4"},
{"----"},
{"----"},
{"ying1"},
{"----"},
{"----"},
{"guan4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"cuo2"},
{"lu4"},
{"----"},
{"----"},
{"ji3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"jun1"},
{"zhu3"},
{"----"},
{"----"},
{"mi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi2"},
{"lu4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"she4"},
{"----"},
{"lin2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"mai4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"fu1"},
{"----"},
{"----"},
{"ma2"},
{"----"},
{"mo2"},
{"hui1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"huang2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu3"},
{"li2"},
{"nian2"},
{"----"},
{"hei1"},
{"----"},
{"----"},
{"qian2"},
{"----"},
{"----"},
{"----"},
{"mo4"},
{"----"},
{"----"},
{"dai4"},
{"chu4"},
{"you3"},
{"----"},
{"yi1"},
{"xia2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qing2"},
{"----"},
{"li2"},
{"----"},
{"du2"},
{"can3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"an4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"zhi3"},
{"----"},
{"fu2"},
{"fu3"},
{"----"},
{"min3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"yuan2"},
{"----"},
{"tuo2"},
{"ding3"},
{"----"},
{"nai4"},
{"----"},
{"----"},
{"gu3"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"pi2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"shu3"},
{"----"},
{"fen2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"you4"},
{"----"},
{"----"},
{"wu2"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"xi1"},
{"----"},
{"yan3"},
{"----"},
{"bi2"},
{"----"},
{"qiu2"},
{"han1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"qi2"},
{"ji1"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"chi3"},
{"chen4"},
{"----"},
{"----"},
{"ju3"},
{"ling2"},
{"pao2"},
{"tiao2"},
{"zi1"},
{"yin2"},
{"yu3"},
{"chuo4"},
{"qu3"},
{"wo4"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"----"},
{"long2"},
{"gong1"},
{"kan1"},
{"----"},
{"----"},
{"----"},
{"gui1"},
{"yue4"},
{"----"},
{"----"},
{"----"}
};


// Simple Variables
char aPinyin[256];

/* Init the HW layer */
void ct_hwpinyin_init()
{
	PA_CopyText(aPinyin,"");
}

/* Gets a char * to the pinyin of a hanzi */
char* ct_hwpinyin_getPinyin(u16 hanzi)
{
   int hanziVal = hanzi - 19968; // 4e00
   PA_CopyText(aPinyin,gfx_pinyin[hanziVal]);
   return aPinyin;
}

/* Gets a char * to the meaning of a hanzi */
char* ct_hwpinyin_getMeaning(u16 hanzi)
{
   return "";
}

/* Search a hanzi index with the same pinyin */
int ct_hwpinyin_searchPinyin(int now, char* pinyin)
{
   int i;
   if ((now < 0) || (now >= 20900))
   	return -1;
   for (i = now; i < 20900; i++) {
      if (strcmp(gfx_pinyin[i],pinyin) == 0)
      	return i;
   }   
   return -1;
}   
